# Configuring RubyYacht

All configuration in RubyYacht is done through a `RubyYacht.configure` block.
This block allows you to run methods against `RubyYacht::Configuration::DSL` to
add projects and define hooks. Once the block is done, all the changes are added
into the shared `RubyYacht.configuration` object. You can run arbitrary code
inside this block, but take note that the `RubyYacht.configuration` object will
not be updated inside this block.

# Example

You can see [configuration_sample.rb](https://github.com/brownleej/ruby-yacht/blob/master/doc/configuration_sample.rb)
for an example of what a full configuration file would look like.

# Adding Projects

Inside a `configure` block, you can call `project :apollo` to add a project
named `apollo`. The `project` method takes a block, which allows you to run
methods against `RubyYacht::Project::DSL` to fill in the details of your
project.

### Project Fields

You can provide the following fields.

<table>
  <tr>
    <th>Name</th>
    <th>Description</th>
    <th>Example</th>
    <th>Default</th>
  </tr>
  <tr>
    <td>system_prefix</td>
    <td>
      The prefix that is prepended before the names of the images and containers
      for this project.
    </td>
    <td><code>system_prefix :foo</code></td>
    <td>None; this is required.</td>
  </tr>
  <tr>
    <td>repository</td>
    <td>
      The host name for the code repository that holds the apps.
    </td>
    <td><code>repository 'github.com'</code></td>
    <td>None; this is required.</td>
  </tr>
  <tr>
    <td>repository_protocol</td>
    <td>
      <p>The protocol that we use to check out the repositories.</p>
      <p>This can be `ssh`, `http`, or `https`.</p>
    </td>
    <td><code>repository_protocol 'https'</code></td>
    <td>ssh</td>
  </tr>
  <tr>
    <td>check_out_locally</td>
    <td>
      Whether we should check out the code on the host system and map that
      directory into the docker container. If this is set to false, the code
      will not be easily accessible from the host system.
    </td>
    <td>
      <code>check_out_locally # If you want to set it to true.</code>
    </td>
    <td>
      False
    </td>
  </tr>
  <tr>
    <td>primary_app</td>
    <td>
      The name of the primary app that should be served from the main domain.
      If this is nil, then the main domain will server a landing
      page with a list of the apps, and the apps themselves will be accessed
      through subdomains.
    </td>
    <td>
      <code>primary_app :mars</code>
    </td>
    <td>
      nil
    </td>
  </tr>
</table>

Inside a project block, you also define databases, DNS servers, web servers,
and apps, as described below.

### Plugin-Specific Fields

If you have loaded the Rails plugin, which is loaded by default, the project
will also have the following fields:


<table>
  <tr>
    <th>Name</th>
    <th>Description</th>
    <th>Example</th>
    <th>Default</th>
  </tr>
  <tr>
    <td>rails_environment</td>
    <td>
      The environment for the Rails apps.
    </td>
    <td><code>rails_environment 'development'</code></td>
    <td>development</td>
  </tr>
  <tr>
    <td>rails_secret_key_base</td>
    <td>
      <p>
        The key for signing sessions and other app secrets in the Rails apps.
      </p>
      <p>
        You will probably want to keep this outside of your configuration
        scripts and outside of source control, so that it is kept secure and can
        be set to different values in different environments.
      </p>
    </td>
    <td><code>rails_secret_key_base 'abc123'</code></td>
    <td>None; this is required</td>
  </tr>
  <tr>
    <td>rails_excluded_gem_groups</td>
    <td>
      The gem groups from the Gemfile that should not be installed on the
      server.
    </td>
    <td><code>rails_excluded_gem_groups %w(development test)</code></td>
    <td>An empty array</td>
  </tr>
</table>

For more information about working with the default plugins, see
[Default Plugins](#Default_Plugins) below.

# Adding Apps

Inside the project DSL, you can call `app :rails, :mars` to add a Rails app
called `mars`. You can also call `rails_app :mars` to do the same thing. Both
forms take a block, which allows you to call methods from `RubyYacht::App::DSL`.

If you are using a different app type, you can supply that type instead of
`rails`, but the app type must be defined through a
[plugin](http://www.rubydoc.info/gems/ruby_yacht/file/doc/plugins.md).

You can call the `app` method multiple times to add multiple databases.

### App Fields

You can provide the following fields in the app DSL:

<table>
  <tr>
    <th>Name</th>
    <th>Description</th>
    <th>Example</th>
    <td>Default</td>
  </tr>
  <tr>
    <td>repository_name</td>
    <td>
      <p>
        The name of the code repository holding this app. This is relative to
        the project's repository.
      </p>
      <p>
        If this is nil, then the scripts will create a new app instead of trying
        to check out an existing one. When combined with the `check_out_locally`
        flag on the project, this can be a good way to bootstrap new apps.
      </p>
    </td>
    <td><code>repository_name 'brownleej/mars'</code></td>
    <td>
      nil
    </td>
  </tr>
  <tr>
    <td>database_name</td>
    <td>
      The name of the database this app uses. If this is not provided, the app
      will not have any database configured automatically.
    </td>
    <td><code>database_name :apollo</code></td>
    <td>nil</td>
  </tr>
  <tr>
    <td>port</td>
    <td>
      <p>The port the app listens on.</p>
      <p>
        <strong>Note:</strong> The app servers will only be accessible within
        the docker network; they will not be directly accessible from the host
        machine or the outside world. For this reason, all of your apps can
        share the same port without causing any conflicts.
      </p>
    </td>
    <td><code>port 3000</code></td>
    <td>8080</td>
</table>

# Adding Databases

Inside the project DSL, you can call `database :mysql, :apollo` to add a MySQL
database called `apollo`. You can also call `mysql_database :apollo` to do the
same thing. Both forms take a block, which allows you to call methods from
`RubyYacht::Database::DSL`.

You can call the `database` method multiple times to add multiple databases.

If you are using a different database type, you can supply that type instead of
`mysql`, but the database type must be defined through a
[plugin](http://www.rubydoc.info/gems/ruby_yacht/file/doc/plugins.md).

### Database Fields

<table>
  <tr>
    <th>Name</th>
    <th>Description</th>
    <th>Example</th>
    <td>Default</td>
  </tr>
  <tr>
    <td>host</td>
    <td>
      The name of the host that the database server is on. If you provide
      `localhost`, this will create a database image and a database container
      for the database. If you provide any other value, this will not create
      any database image or container, and will only use the database config
      to point the apps toward the correct external server.
    </td>
    <td><code>host 'db1.test.com'</code></td>
    <td>None; this is required.</td>
  </tr>
  <tr>
    <td>username</td>
    <td>
      The name of the user that the apps will use to connect to the database.
    </td>
    <td><code>username 'apollo'</code></td>
    <td>None; this is required.</td>
  </tr>
  <tr>
    <td>password</td>
    <td>
      The password that the apps will use to connect to the database.
    </td>
    <td><code>password 'testpass'</code></td>
    <td>None; this is required.</td>
  </tr>
  <tr>
    <td>port</td>
    <td>
      The port that the database server listens on.
    </td>
    <td><code>port 3377</code></td>
    <td>
      In general, this does not have a default value. For MySQL databases,
      though, this will be set to 3306 by default.
    </td>
  </tr>
  <tr>
    <td>container_label</td>
    <td>
      The label for the images and containers for this database. For instance,
      if your project prefix is `apollo` and the container label for the
      database is `mysql`, the image and container for the database will be
      called `apollo-mysql`.
    </td>
    <td><code>container_label :mysql</code></td>
    <td>database</td>
  </tr>
</table>

# Adding Web Servers

The app containers will not publish their ports to the host machine or the
outside world, so you need to define web servers that serve as a proxy for them.
Inside the project DSL, you can call `web_server :nginx` to add an Nginx
web server. You can also call `nginx_web_server` to do the
same thing. Both forms take a block, which allows you to call methods from
`RubyYacht::WebServer::DSL`.

By default, the web server will be called `web`, and its image name and
container name will have the format `apollo-web`, where `apollo` is the system
prefix for the project. If you want to use a different name, you can pass this
as an argument to `web_server` or `nginx_web_server`.

You can call the `web_server` method multiple times to add multiple servers.
The scripts will try to start all of the web servers for your projects at once,
which will cause conflicts if they are trying to listen on the same port. This
is also a potential problem if you have multiple projects in the same
configuration file, since each project will need its own web server.

If you are using a different web server type, you can supply that type instead
of `nginx`, but the server type must be defined through a
[plugin](http://www.rubydoc.info/gems/ruby_yacht/file/doc/plugins.md).

<table>
  <tr>
    <th>Name</th>
    <th>Description</th>
    <th>Example</th>
    <td>Default</td>
  </tr>
  <tr>
    <td>domain</td>
    <td>
      The main domain for this server. Different apps will be added as
      subdomains of this main domain.
    </td>
    <td><code>domain 'test.com'</code></td>
    <td>None; this is required.</td>
  </tr>
  <tr>
    <td>port</td>
    <td>
      The port that this server listens on. This is the port on the
      *host machine* that the requests will come into. Inside the container, the
      server will always listen on port 80, and it will be mapped to this port=
      on the host.
    </td>
    <td><code>port 8080</code></td>
    <td>80</td>
  </tr>
</table>

# DNS Server Config

If your network has custom DNS servers, you can call `dns_server` inside of a
project block to define your DNS server config. The `dns_server` method takes
no arguents, but takes a block which allows you to call methods from
`RubyYacht::DnsServer::DSL`. 

You should only call the `dns_server` once for a given project.

### DNS Server Fields


<table>
  <tr>
    <th>Name</th>
    <th>Description</th>
    <th>Example</th>
    <td>Default</td>
  </tr>
  <tr>
    <td>server</td>
    <td>
      The hostname or IP address for the DNS server. You can call this
      multiple times to add multiple DNS servers
    </td>
    <td><code>server 'dns.test.com'</code></td>
    <td>Empty; no DNS servers</td>
  </tr>
  <tr>
    <td>search_domain</td>
    <td>
      The default search domains for server names. You can call this multiple
      times to add multiple search domains.
    </td>
    <td><code>search_domain 'db.test.com'</code></td>
    <td>Empty; no DNS search domains</td>
  </tr>
</table>

# Local Configuration

You may want to have some of your configuration stored in a local file, outside
of source control, to handle host-specific settings. For instance, you may have
different users using different remote databases, or you may want to set the
Rails environment differently in production than in testing. You can do this
through two steps. First, you can call `add_local_config` to load the
configuration from the YAML file:

        RubyYacht.configure do
          add_local_config File.join(File.dirname(__FILE__), 'config.yml')
        end

This has to be done in a separate configuration block, because the loaded config
will not be available until the configuration block is over.

Inside your main configuration block, you can then call `copy_local_config` to
set the fields in the DSL based on the local config:

        project :apollo do
          system_prefix :apollo

          copy_local_config :rails_environment, :rails_secret_key_base
        end

You may also want to have some of your configuration fields grouped together to
keep the config file neater. For instance, you can have a database section of
the config YML file that is structured like this:

        :database:
          :apollo:
            :host: db1.apollo.com
            :username: apollo
            :password: ;kljfadsl
            :name: apollo_production

And then inside your project config, you could call:

        database :apollo do
          copy_local_config :host, :username, :password, :name, from: 'database.apollo'
        end

# Default Plugins

By default, RubyYacht comes with three plugins, defined under the
`RubyYacht::Plugins` namespace:

* Rails, for defining app servers for Rails apps
* MySQL, for defining MySQL database servers
* Nginx, for defining Nginx web servers.

If you want to unload the default plugins, you can call
`RubyYacht.configuration.clear` before defining the rest of your configuration.
You can then load individual plugins by calling, for instance,
`RubyYacht::Plugins::MySQL.load`.

# Miscellaneous Configuration

The `disable_docker_machine` flag on `RubyYacht.configuration` forces the scripts
to act as though docker-machine is not installed. This can be helpful if you are
running a Docker for Mac or Docker for Windows beta alongside a docker-machine-based
installation.