# Plugins

RubyYacht's plugin engine allows you to define your own server types, and add
custom hooks for what happens when building images and running containers.

You can define server types and hooks inside of a `RubyYacht.configure` block. Like
other parts of the configuration, they will not be committed until the
`configure` block is over. The app and database DSLs require that the server
types be committed, so you should add server types in a separate configuration
block from the one where you define your projects.

# Adding Server Types

You can call `server_type :mongo` to define a server type called `mongo`. The
`server_type` method takes a block, which allows you to call methods from
`RubyYacht::ServerType::DSL`.

### Server Type Fields

You can add the following fields to fill out the server type:

<table>
  <tr>
    <th>Name</th>
    <th>Description</th>
    <th>Example</th>
    <th>Default</th>
  </tr>
  <tr>
    <td>container_type</td>
    <td>
      The type of container that this server type applies to. This can be `app`,
      `database`, or `web`.
    </td>
    <td>
      <code>container_type :app</code>
    </td>
    <td>None; this is required.</td>
  </tr>
  <tr>
    <td>baseline_image</td>
    <td>
      <p>
        The docker image that we should use as the baseline for the images for
        these servers.
      </p>
      <p>
        This is currently ignored for database images, since those are built off
        of the app dependencies image.
      </p>
    </td>
    <td>
      <code>baseline_image 'ruby:2.3'</code>
    </td>
    <td>
      None; this is required.
    </td>
  </tr>
  <tr>
    <td>project_attribute</td>
    <td>
      <p>This defines an attribute that can be set on a project.</p>
      <p>
        This can be set on any project once the server type has been loaded.
      </p>
      <p>
        This accepts hash of options. The currently accepted options are `name`
        and `default`. The full name of the attribute will be prefixed with the
        name of the server type, to keep server types from defining conflicting
        attributes.
      </p>
    </td>
    <td><code>project_attribute name: :environment, default: 'staging'</code></td>
    <td>None; no custom project attributes</td>
  </tr>
  <tr>
    <td>server_attribute</td>
    <td>
      <p>This defines an attribute that can be set on a server.</p>
      <p>
        This can be set on any server of the matching type once the server type
        has been loaded.
      </p>
      <p>
        This accepts hash of options. The currently accepted options are `name`
        and `default`. The full name of the attribute will be prefixed with the
        name of the server type, to keep server types from defining conflicting
        attributes.
      </p>
    </td>
    <td><code>server_attribute name: :key</code></td>
    <td>None; no custom server attributes</td>
  </tr>
  <tr>
    <td>server_default</td>
    <td>
      <p>
        This defines a default value for an existing attribute on servers of
        this type.
      </p>
      <p>
        This accepts hash of options. The keys in this hash are the names of the
        attributes, and the values are the default values.
      </p>
      <p>
        You can call this multiple times, and it will add the new defaults to
        the list.
      </p>
    </td>
    <td><code>server_default port: 3000</code></td>
    <td>None; no custom default values.</td>
  </tr>
</table>

# Adding Hooks

To define a hook, you can call `before`, `during`, or `after`, followed by an
event type. A before-hook is run before the event happens, an after-hook is run
after the event happens, and a during-hook provides the logic for making an
event happen.

These methods also take a block which allow you to call methods from
`RubyYacht::Hook::DSL`.

### Event Types

These are the event types that you can hook into:

<table>
  <tr>
    <th>Name</th>
    <th>Description</th>
  </tr>
  <tr>
    <td>startup</td>
    <td>An app server is starting up.</td>
  </tr>
  <tr>
    <td>build_checkout</td>
    <td>
      The code for an app is being checked out when building an app image.
    </td>
  </tr>
  <tr>
    <td>build_new_app</td>
    <td>
      An app is being created from scratch rather than checked out from a
      repository.
    </td>
  </tr>
  <tr>
    <td>install_libraries</td>
    <td>The libraries for a server are being installed.</td>
  </tr>
  <tr>
    <td>initialize_app_environment</td>
    <td>
      The envrionment variables for the app-dependencies image are being set.
    </td>
  </tr>
  <tr>
    <td>create_databases</td>
    <td>The databases for a database server are being created.</td>
  </tr>
  <tr>
    <td>load_database_seeds</td>
    <td>The seed data for the app is being loaded on the database server.</td>
  </tr>
  <tr>
    <td>add_project_landing</td>
    <td>
      We are adding the configuration for a project's landing page to a web
      server.
    </td>
  </tr>
  <tr>
    <td>add_app_config</td>
    <td>We are adding the configuration for a single app to a web server.</td>
  </tr>
  <tr>
    <td>cleanup</td>
    <td>This is run at the end of the build process for each image.</td>
  </tr>
</table>

### Fields

The hook DSL allows you to set the following fields:

<table>
  <tr>
    <th>Name</th>
    <th>Description</th>
    <th>Example</th>
    <th>Default</th>
  </tr>
  <tr>
    <td>container_type</td>
    <td>The name of the container type that this hook applies to</td>
    <td><code>container_type :app</code></td>
    <td>None; this is required</td>
  </tr>
  <tr>
    <td>app_server_type</td>
    <td>The name of the app server type that this hook applies to</td>
    <td><code>app_server_type :rails</code></td>
    <td>all, meaning all app server types</td>
  </tr>
  <tr>
    <td>database_server_type</td>
    <td>The name of the database server type that this hook applies to</td>
    <td><code>database_server_type :mysql</code></td>
    <td>all, meaning all database server types</td>
  </tr
  <tr>
    <td>web_server_type</td>
    <td>The name of the web server type that this hook applies to</td>
    <td><code>web_server_type :nginx</code></td>
    <td>all, meaning all web server types</td>
  </tr
  <tr>
    <td>script_folder</td>
    <td>
      The path to the folder in the host machine containing files to copy for
      this hook.
    </td>
    <td><code>script_folder './scripts'</code></td>
    <td>The working directory</td>
  </tr>
</table>

You can constrain a hook by multiple types of servers at once.. For instance,
you can set the app_server_type to :rails and the database_server_type to :mysql
to specify that the hook only applies to Rails apps backed by a MySQL database.
By default, hooks apply to all server types. You also have to specify the
container_type to say whether the hook is triggered by events on app images,
database images, or web server images. For instance,
both app images and database images have a `startup` event, and they'll need
different behaviors defined.

You can define the behaviors for the hooks with these methods:

<table>
  <tr>
    <th>Method Name</th>
    <th>Description</th>
    <th>Example</th>
  </tr>
  <tr>
    <td>command</td>
    <td>The shell command that should be run when the hook executes</td>
    <td><code>command 'cp /var/docker/database.yml /var/code/config/'</code></td>
  </tr>
  <tr>
    <td>copy_file</td>
    <td>
      The name of a file that should be copied from the host machine into the
      image. This is relative to the hook's script folder.
    </td>
    <td><code>copy_file 'setup.rb'</code></td>
  </tr>
  <tr>
    <td>run_script</td>
    <td>
      This specifies that this hook runs a script from the host machine. This
      will set the `copy_file` and the `command`.
    </td>
    <td><code>run_script 'setup.rb'</code></td>
  </tr>
  <tr>
    <td>set_environment_variable</td>
    <td>
      This sets an environment variable in the image. The value can be set in
      either a block or a literal value passed as the second parameter.
    </td>
    <td>
      <code>
        set_environment_variable 'RAILS_ENV' do
          @project.rails_environment
        end
      </code>
    </td>
  </tr>
</table>

### Setting Defaults

Your plugins will likely have many hooks that are using some of the same
settings, so you can set defaults for them by calling `add_hooks`. This method
takes a hash containing options for the hooks, and a block for adding them. Any
hooks added in the block will have those options set.

The supported options are `script_folder`, `app_server_type`,
`database_server_type`, `web_server_type`, and `container_type`.