require 'spec_helper'

RSpec.describe RubyYacht::App do
  describe "DSL" do
    let(:name) { :test_app }
    let(:app) { RubyYacht::App::DSL.new(:test, name).run(@builder).create_object }

    before do
      RubyYacht.configuration.clear
      RubyYacht.configure do
        server_type :test do
          container_type :app
          baseline_image 'ubuntu'
        end
      end
    end
    
    it "creates an app with all fields" do
      @builder = Proc.new do
        repository_name 'brownleej/test-app'
        database_name :database
        port 3000
      end
      expect(app.name).to eq :test_app
      expect(app.repository_name).to eq 'brownleej/test-app'
      expect(app.database_name).to eq :database
      expect(app.server_type).to eq :test
      expect(app.port).to eq 3000
    end

    it "defaults the repository name to nil" do
      @builder = Proc.new do
        port 3000
        database_name :database
      end
      expect(self.app.repository_name).to be_nil
    end

    it "does not require the database name" do
      @builder = Proc.new do
        repository_name 'brownleej/test-app'
        port 3000
      end
      expect(app.database_name).to be_nil
    end

    it "requires a defined server type" do
      @builder = Proc.new do
        server_type :invalid
        repository_name 'brownleej/test-app'
        port 3000
        database_name :database
      end
      expect do
        self.app
      end.to raise_exception("RubyYacht::App::DSL has invalid app server type `invalid`")
    end

    it "requires a defined server type for apps" do
      RubyYacht.configure do
        server_type :sqlite do
          container_type :database
          baseline_image 'ubuntu'
        end
      end

      @builder = Proc.new do
        server_type :sqlite
        repository_name 'brownleej/test-app'
        port 3000
        database_name :database
      end
      expect do
        self.app
      end.to raise_exception("RubyYacht::App::DSL has invalid app server type `sqlite`")
    end

    it "defaults the port to 8080" do
      @builder = Proc.new do
        repository_name 'brownleej/test-app'
        database_name :database
      end
      expect(app.port).to eq 8080
    end
  end

  describe 'project' do
    before do
      RubyYacht.configuration.clear
      RubyYacht.configure do
        server_type :test do
          container_type :app
          baseline_image 'ubuntu'
        end
      end
      RubyYacht.configure do
        project :project1 do
          repository 'github.com'
          system_prefix :project1

          test_app :app1 do
            repository_name 'app1'
          end

          test_app :app2 do
            repository_name 'app2'
          end
        end
        
        project :project2 do
          repository 'github.com'
          system_prefix :project2

          test_app :app3 do
            repository_name 'app3'
          end
        end
      end
    end

    it 'finds the project that contains the app' do
      app2 = RubyYacht.configuration.projects[0].apps[1]
      expect(app2.project.name).to eq :project1

      app3 = RubyYacht.configuration.projects[1].apps[0]
      expect(app3.project.name).to eq :project2
    end
  end

  describe "container_name" do
    before do
      @project = RubyYacht::Project.new
      @project.name = :test_project
      @project.system_prefix = :tests

      @app = RubyYacht::App.new
      @app.name = 'app1'
      @project.apps = [@app]
      RubyYacht.configuration.projects << @project
    end

    it "combines the project's prefix with the app's name" do
      expect(@app.container_name).to eq 'tests-app1'
    end

    context "with the same name as the project's system_prefix" do
      it "is just the project's system prefix" do
        @app.name = :tests

        expect(@app.container_name).to eq 'tests'
      end
    end
  end

  describe "database" do
    before do
      @project = RubyYacht::Project.new
      @project.name = :test_project
      @project.system_prefix = :tests

      database1 = RubyYacht::Database.new
      database1.name = 'main_db'
      database1.host = 'localhost'

      database2 = RubyYacht::Database.new
      database2.name = 'second_db'
      database2.host = 'db2.test.com'

      @project.databases = [database1, database2]

      @app = RubyYacht::App.new
      @app.name = 'app1'
      @app.database_name = 'second_db'

      @project.apps = [@app]
      RubyYacht.configuration.projects << @project
    end

    it "is the database with the app's database_name" do
      database = @app.database
      expect(database).not_to be_nil
      expect(database.host).to eq 'db2.test.com'
    end

    context "with no matching database" do
      before do
        @project.databases.pop
      end

      it "is nil" do
        expect(@app.database).to be_nil
      end
    end
  end
end