require 'spec_helper'
require 'ruby_yacht'

describe RubyYacht::Configuration do
  before do
    RubyYacht.configuration.clear
  end

  describe "reading configuration" do
    describe "fetch_hooks" do
      before do
        RubyYacht.configure do
          server_type :django do
            container_type :app
            baseline_image 'ubuntu'
          end
          server_type :php do
            container_type :app
            baseline_image 'ubuntu'
          end

          server_type :postgres do
            container_type :database
            baseline_image 'ubuntu'
          end
          server_type :mongo do
            container_type :database
            baseline_image 'ubuntu'
          end

          server_type :nginx do
            container_type :web
            baseline_image 'ubuntu'
          end

          server_type :apache do
            container_type :web
            baseline_image 'ubuntu'
          end
        end

        RubyYacht.configure do
          add_hooks container_type: :app, app_server_type: :django, database_server_type: :postgres, script_folder: '.' do
            before :startup do
              run_script :bash, 'app-a.bash'
            end

            before :install_libraries do
              run_script :bash, 'app-b.bash'
            end
            
            before :startup do
              app_server_type :php
              run_script :bash, 'app-c.bash'
            end

            before :startup do
              database_server_type :mongo
              run_script :bash, 'app-d.bash'
            end

            before :startup do
              database_server_type :all
              run_script :bash, 'app-e.bash'
            end

            before :startup do
              app_server_type :all
              run_script :bash, 'app-f.bash'
            end
          end

          add_hooks container_type: :database, app_server_type: :django, database_server_type: :postgres, script_folder: '.' do
            before :startup do
              run_script :bash, 'database-a.bash'
            end

            before :install_libraries do
              run_script :bash, 'database-b.bash'
            end
            
            before :startup do
              database_server_type :mongo
              run_script :bash, 'database-c.bash'
            end

            before :startup do
              app_server_type :php
              run_script :bash, 'database-d.bash'
            end

            before :startup do
              app_server_type :all
              run_script :bash, 'database-e.bash'
            end

            before :startup do
              database_server_type :all
              run_script :bash, 'database-f.bash'
            end
          end

          add_hooks container_type: :web, web_server_type: :nginx, script_folder: '.' do
            before :startup do
              run_script :bash, 'web-a.bash'
            end

            before :startup do
              web_server_type :all
              run_script :bash, 'web-b.bash'
            end

            before :startup do
              web_server_type :apache
              run_script :bash, 'web-c.bash'
            end

            before :install_libraries do
              run_script :bash, 'web-d.bash'
            end
          end
        end

        RubyYacht.configure do
          project :test_project do
            repository 'github.com'
            system_prefix :test_project

            postgres_database :db1 do
              host :localhost
              username 'test'
              password 'test'
              port 5432
            end

            mongo_database :db2 do
              host :localhost
              username 'test'
              password 'test'
              port 27017
            end

            django_app :app1 do
              database_name :db1
              repository_name 'test/app1'
            end

            php_app :app2 do
              database_name :db2
              repository_name 'test/app2'
            end

            nginx_web_server do
              domain 'test.com'
            end
          end
        end
      end

      let(:hooks) { RubyYacht.configuration.fetch_hooks(server, :startup) }
      let(:filenames) { hooks.collect { |hook| File.basename(hook.behaviors.first.file_path) } }

      context "with an app" do
        let(:server) { RubyYacht.configuration.projects.first.apps.first }
        it "finds hooks for that app" do
          expect(filenames).to include 'app-a.bash'
        end

        it "finds hooks for all apps" do
          expect(filenames).to include 'app-f.bash'
        end

        it "does not find hooks for other apps" do
          expect(filenames).not_to include 'app-c.bash'
        end

        it "finds hooks for all databases for that app" do
          expect(filenames).to include 'app-e.bash'
        end

        it "does not find hooks for other databases for that app" do
          expect(filenames).not_to include 'app-d.bash'
        end

        it "does not find hooks for other events" do
          expect(filenames).not_to include 'app-b.bash'
        end

        it "does not find hooks for database containers" do
          expect(filenames).not_to include 'database-a.bash'
        end
      end

      context "with a database" do
        let(:server) { RubyYacht.configuration.projects.first.databases.first }

        it "finds hooks for that databases" do
          expect(filenames).to include 'database-a.bash'
        end

        it "finds hooks for all databases" do
          expect(filenames).to include 'database-f.bash'
        end

        it "does not find hooks for other databases" do
          expect(filenames).not_to include 'database-c.bash'
        end

        it "finds hooks for all apps for that database" do
          expect(filenames).to include 'database-e.bash'
        end

        it "does not find hooks for other apps for that database" do
          expect(filenames).not_to include 'database-d.bash'
        end

        it "does not find hooks for other events" do
          expect(filenames).not_to include 'database-b.bash'
        end
      end

      context 'with a web server' do
        let(:server) { RubyYacht.configuration.projects.first.web_servers.first }

        it "finds hooks for that type" do
          expect(filenames).to include 'web-a.bash'
        end

        it "finds hooks for all types" do
          expect(filenames).to include 'web-b.bash'
        end

        it "does not find hooks for other types" do
          expect(filenames).not_to include 'web-c.bash'
        end

        it "does not find hooks for other events" do
          expect(filenames).not_to include 'web-d.bash'
        end
      end

      context "with an invalid type" do
        it "is an empty array" do
          expect(RubyYacht.configuration.fetch_hooks(:test, :startup)).to eq []
        end
      end
    end

    describe "find_server_type" do
      before do
        RubyYacht.configure do
          server_type :generic do
            container_type :app
            baseline_image 'ubuntu'
          end

          server_type :debian do
            container_type :app
            baseline_image 'debian'
          end
        end
      end

      it "returns the app type with that name" do
        type = RubyYacht.configuration.find_server_type(:debian)
        expect(type.baseline_image).to eq 'debian'
      end
    end
  end

  describe "dsl" do
    let(:project_creation_block) do
      Proc.new do
        project :project1 do
          system_prefix :a
          repository "github.com"
        end

        project :project2 do
          system_prefix :b
          repository "github.com"
        end
      end
    end

    let(:hook_creation_block) do
      Proc.new do
        add_hooks container_type: :app, app_server_type: :configuration_test, script_folder: './scripts' do
          before :startup do
            run_script :ruby, 'update_config.rb'
          end
        end
      end
    end

    let(:server_type_creation_block) do
      Proc.new do
        server_type :second_configuration_test do
          container_type :app
          baseline_image 'debian'
          server_attribute :test_attribute
        end
      end
    end

    before do
      RubyYacht.configure do
        server_type :configuration_test do
          container_type :app
          baseline_image 'ubuntu'
        end
        server_type :configuration_test_db do
          container_type :database
          baseline_image 'ubuntu'
        end
      end
    end

    it "can add projects" do
      configuration = RubyYacht::Configuration::DSL.new.run(project_creation_block).create_object
      expect(configuration.projects.count).to eq 2
      expect(configuration.projects[0].name).to eq :project1
      expect(configuration.projects[1].name).to eq :project2
    end

    it "can add hooks" do
      configuration = RubyYacht::Configuration::DSL.new.run(hook_creation_block).create_object
      expect(configuration.hooks.count).to eq 1
      expect(configuration.hooks[0].event_time).to eq :before
      expect(configuration.hooks[0].event_type).to eq :startup
      expect(configuration.hooks[0].behaviors.first.file_path).to eq './scripts/update_config.rb'
    end

    it "can add server types" do
      configuration = RubyYacht::Configuration::DSL.new.run(server_type_creation_block).create_object
      expect(configuration.server_types.count).to eq 1
      expect(configuration.server_types[0].name).to eq :second_configuration_test
      expect(configuration.server_types[0].project_attributes.count).to eq 0
      expect(configuration.server_types[0].server_attributes.count).to eq 1
    end

    it 'can add local config' do
      configuration = RubyYacht::Configuration::DSL.new.run(Proc.new do
        add_local_config File.join(File.dirname(__FILE__), '..', 'fixtures', 'local_config.yml')
      end)

      expect(configuration.local_config).to eq(
        system_prefix: 'jupiter',
        check_out_locally: true,
        rails_environment: 'production',
        database: {
          host: 'localhost',
          name: 'jupiter_testing',
          username: 'jupiter',
          password: 'jupiterpass'
        }
      )
    end

    it "can add `during` hooks" do
      configuration = RubyYacht::Configuration::DSL.new.run(Proc.new do
        add_hooks container_type: :app, app_server_type: :configuration_test, script_folder: './scripts' do
          during :startup do
            run_script :ruby, 'start_app.rb'
          end
        end
      end).create_object
      expect(configuration.hooks.count).to eq 1
      expect(configuration.hooks[0].event_time).to eq :during
      expect(configuration.hooks[0].event_type).to eq :startup
      expect(configuration.hooks[0].app_server_type).to eq :configuration_test
      expect(configuration.hooks[0].database_server_type).to eq :all
      expect(configuration.hooks[0].container_type).to eq :app
      expect(configuration.hooks[0].behaviors.first.file_path).to eq './scripts/start_app.rb'
    end

    it "can add `after` hooks" do
      configuration = RubyYacht::Configuration::DSL.new.run(Proc.new do
        add_hooks container_type: :database, database_server_type: :configuration_test_db, script_folder: './scripts' do
          after :startup do
            run_script :ruby, 'start_app.rb'
          end
        end
      end).create_object
      expect(configuration.hooks.count).to eq 1
      expect(configuration.hooks[0].event_time).to eq :after
      expect(configuration.hooks[0].event_type).to eq :startup
      expect(configuration.hooks[0].app_server_type).to eq :all
      expect(configuration.hooks[0].database_server_type).to eq :configuration_test_db
      expect(configuration.hooks[0].container_type).to eq :database
      expect(configuration.hooks[0].behaviors.first.file_path).to eq './scripts/start_app.rb'
    end

    describe "configure method" do
      it "adds to the list of projects" do
        project = RubyYacht::Project.new
        project.name = :initial_project
        RubyYacht.configuration.projects << project
        RubyYacht.configure(&project_creation_block)

        configuration = RubyYacht.configuration
        expect(configuration.projects.count).to eq 3
        expect(configuration.projects[0].name).to eq :initial_project
        expect(configuration.projects[1].name).to eq :project1
        expect(configuration.projects[2].name).to eq :project2
      end

      it "adds to the list of hooks" do
        hook = RubyYacht::Hook.new
        hook.event_time = :after
        hook.event_type = :restart_application
        RubyYacht.configuration.hooks << hook
        RubyYacht.configure(&hook_creation_block)

        configuration = RubyYacht.configuration
        expect(configuration.hooks.count).to eq 2
      end

      it "adds to the list of server types" do
        RubyYacht.configure(&server_type_creation_block)

        configuration = RubyYacht.configuration
        expect(configuration.server_types.count).to eq 3
        expect(configuration.server_types[0].name).to eq :configuration_test
        expect(configuration.server_types[1].name).to eq :configuration_test_db
        expect(configuration.server_types[2].name).to eq :second_configuration_test
      end

      it "adds to the local config" do
        RubyYacht.configuration.local_config = {system_prefix: 'apollo', rails_secret_key_base: 'ABC1234'}
        RubyYacht.configure do
          add_local_config File.join(File.dirname(__FILE__), '..', 'fixtures', 'local_config.yml')
        end
        
        configuration = RubyYacht.configuration
        expect(configuration.local_config[:system_prefix]).to eq 'jupiter'
        expect(configuration.local_config[:rails_secret_key_base]).to eq 'ABC1234'
      end

      context "with a server type that is already registered" do
        before do
          type = RubyYacht::ServerType.new
          type.name = :second_configuration_test
          type.server_attributes = []
          RubyYacht.configuration.server_types << type
        end

        it "raises an exception" do
          expect(lambda do
            RubyYacht.configure(&server_type_creation_block)
          end).to raise_exception "Server type already registered: second_configuration_test"
        end
      end
    end
  end
end