require 'spec_helper'

describe RubyYacht::Database do
  before do
    RubyYacht.configuration.clear
    RubyYacht.configure do
      server_type :sqlite do
        container_type :database
        baseline_image 'ubuntu'
      end
    end
  end

  describe "dsl" do
    let(:database) { RubyYacht::Database::DSL.new(:sqlite, :apps).run(@builder).create_object }

    it "creates a database with all the fields" do
      @builder = Proc.new do
        host 'db1.test.com'
        username 'test-user'
        password 'test-pass'
        port 1237
        container_label :database
      end

      expect(database.host).to eq 'db1.test.com'
      expect(database.name).to eq :apps
      expect(database.server_type).to eq :sqlite
      expect(database.username).to eq 'test-user'
      expect(database.password).to eq 'test-pass'
      expect(database.port).to eq 1237
      expect(database.container_label).to eq :database
    end

    it "requires the host" do
      @builder = Proc.new do
        username 'test-user'
        password 'test-pass'
        port 1237
        container_label :database
      end

      expect { database }.to raise_exception "Missing required attribute host for RubyYacht::Database::DSL"
    end

    it "requires the username" do
      @builder = Proc.new do
        host 'db1.test.com'
        password 'test-pass'
        port 1237
        container_label :database
      end

      expect { database }.to raise_exception "Missing required attribute username for RubyYacht::Database::DSL"
    end

    it "requires the password" do
      @builder = Proc.new do
        host 'db1.test.com'
        username 'test-user'
        port 1237
        container_label :database
      end

      expect { database }.to raise_exception "Missing required attribute password for RubyYacht::Database::DSL"
    end

    it "uses 'database' as the default container label" do
      @builder = Proc.new do
        host 'db1.test.com'
        username 'test-user'
        password 'test-pass'
        port 1237
      end

      expect(database.container_label).to eq :database
    end

    it "requires a defined server type" do
      @builder = Proc.new do
        host 'db1.test.com'
        username 'test-user'
        password 'test-pass'
        port 1237
        container_label :database
        server_type :invalid
      end

      expect { database }.to raise_exception "RubyYacht::Database::DSL has invalid database server type `invalid`"
    end

    it "requires a defined server type for a database" do
      RubyYacht.configure do
        server_type :generic do
          container_type :app
          baseline_image 'ruby:2.3'
        end
      end

      @builder = Proc.new do
        host 'db1.test.com'
        username 'test-user'
        password 'test-pass'
        port 1237
        container_label :database
        server_type :generic
      end

      expect { database }.to raise_exception "RubyYacht::Database::DSL has invalid database server type `generic`"
    end
  end

  describe "local?" do
    it "is true when the host is localhost" do
      database = RubyYacht::Database.new
      database.host = 'localhost'
      expect(database).to be_local
    end

    it "is false when the host is not localhost" do
      database = RubyYacht::Database.new
      database.host = 'db1.test.com'
      expect(database).not_to be_local
    end
  end

  describe 'project' do
    before do
      RubyYacht.configuration.clear
      RubyYacht.configure do
        server_type :test do
          container_type :database
          baseline_image 'ubuntu'
        end
      end
      RubyYacht.configure do
        project :project1 do
          repository 'github.com'
          system_prefix :project1

          test_database :db1 do
            host :localhost
            username :test
            password :test
            port 1237
          end

          test_database :db2 do
            host :localhost
            username :test
            password :test
            port 1237
          end
        end
        
        project :project2 do
          repository 'github.com'
          system_prefix :project2

          test_database :db3 do
            host :localhost
            username :test
            password :test
            port 1237
          end
        end
      end
    end

    it 'finds the project that contains the apps' do
      db2 = RubyYacht.configuration.projects[0].databases[1]
      db3 = RubyYacht.configuration.projects[1].databases[0]

      expect(db2.project.name).to eq :project1
      expect(db3.project.name).to eq :project2
    end
  end

  describe 'apps' do
    before do
      @project = RubyYacht::Project.new
      @project.name = :test_project
      @project.system_prefix = :tests

      @database = RubyYacht::Database.new
      @database.name = :db1
      @database.container_label = 'mysql'

      @project.databases = [@database]
      @project.apps = []

      app = RubyYacht::App.new
      app.name = :app1
      app.database_name = :db1
      @project.apps << app

      app = RubyYacht::App.new
      app.name = :app2
      app.database_name = :db1
      @project.apps << app

      app = RubyYacht::App.new
      app.name = :app3
      app.database_name = :db2
      @project.apps << app

      RubyYacht.configuration.projects << @project
    end

    it 'is the apps from the project that have the same database name' do
      expect(@database.apps.map(&:name)).to eq [:app1, :app2]
    end
  end

  describe "container_name" do
    before do
      @project = RubyYacht::Project.new
      @project.name = :test_project
      @project.system_prefix = :tests

      @database = RubyYacht::Database.new
      @database.container_label = 'mysql'

      @project.databases = [@database]
      RubyYacht.configuration.projects << @project
    end

    it "combines the project's prefix with the database's container label" do
      expect(@database.container_name).to eq 'tests-mysql'
    end
  end
end