require 'spec_helper'


module RubyYacht::DSL::Base::ClassMethods
  def clear
    @required_attributes = []
    @all_attributes = []
    @default_values = {}
    @copied_attributes = nil
    @created_type = nil
  end
end

module DSLTestHelpers
  class ThingOne
    attr_accessor :name
    attr_accessor :size
  end

  class ThingTwo
    attr_accessor :name
    attr_accessor :color
  end

  class DSL
    extend RubyYacht::DSL::Base::ClassMethods
    include RubyYacht::DSL::Base
  end

  class SecondDSL
    extend RubyYacht::DSL::Base::ClassMethods
    include RubyYacht::DSL::Base
  end
end

describe RubyYacht::DSL::Base do
  let(:DSL) { DSLTestHelpers::DSL }
  let(:SecondDSL) { DSLTestHelpers::SecondDSL }

  before do
    DSLTestHelpers::DSL.clear
    DSLTestHelpers::SecondDSL.clear
  end

  def run_dsl(&block)
    dsl = DSLTestHelpers::DSL.new
    dsl.run(block)
    dsl
  end

  describe "add_generic_attribute" do
    it "adds the attribute to the list" do
      DSLTestHelpers::DSL.add_generic_attribute :namify, :name do |_|
      end

      expect(DSLTestHelpers::DSL.all_attributes).to eq [:name]
      expect(DSLTestHelpers::DSL.required_attributes).to eq [:name]
      expect(DSLTestHelpers::DSL.default_values).to eq({})
    end

    it "defines a method with the method name provided" do
      capture = nil
      DSLTestHelpers::DSL.add_generic_attribute :namify, :name do | value|
        capture = value
      end

      run_dsl do
        namify 'foo'
      end
      expect(capture).to eq 'foo'
    end

    it "can accept optional fields" do
      DSLTestHelpers::DSL.add_generic_attribute :namify, :name, nil, false do |_|
      end

      expect(DSLTestHelpers::DSL.all_attributes).to eq [:name]
      expect(DSLTestHelpers::DSL.required_attributes).to eq []
    end

    it "can accept fields with default values" do
      DSLTestHelpers::DSL.add_generic_attribute :namify, :name, 'hello' do |_|
      end

      expect(DSLTestHelpers::DSL.all_attributes).to eq [:name]
      expect(DSLTestHelpers::DSL.default_values).to eq(name: 'hello')
    end
  end

  describe "add_attribute" do
    before do
      DSLTestHelpers::DSL.add_attribute :name
    end

    it "defines a method for setting the attribute" do
      dsl = run_dsl do
        name 'hi'
      end
      expect(dsl.instance_eval { @name }).to eq 'hi'
    end

    it "makes the field required" do
      expect(DSLTestHelpers::DSL.required_attributes).to eq [:name]
    end

    it "can accept default values" do
      DSLTestHelpers::DSL.clear
      DSLTestHelpers::DSL.add_attribute :name, 'test'
      dsl = run_dsl {}
      expect(dsl.instance_eval { @name }).to eq 'test'
    end

    it "replaces the value when called multiple times" do
      dsl = run_dsl do
        name 'name1'
        name 'name2'
      end
      expect(dsl.instance_eval { @name }).to eq 'name2'
    end
  end

  describe "add_list" do
    before do
      DSLTestHelpers::DSL.add_list :hat
    end

    it "defines a methods for capturing values in a list" do
      dsl = run_dsl do
        hat 'red'
        hat 'black'
      end

      expect(dsl.instance_eval { @hats }).to eq ['red', 'black']
    end

    it "gives an empty list when the method is not called" do
      dsl = run_dsl {}

      expect(dsl.instance_eval { @hats }).to eq []
    end
  end

  describe "add_boolean" do
    before do
      DSLTestHelpers::DSL.add_boolean :break_the_rules
    end

    it "defines a method for setting the flag to true" do
      dsl = run_dsl do
        break_the_rules
      end
      expect(dsl.instance_eval { @break_the_rules }).to be_truthy
    end

    it "leaves the flag false by default" do
      dsl = run_dsl {}
      expect(dsl.instance_eval { @break_the_rules }).to be_falsey
    end
  end

  describe "add_object" do
    before do
      DSLTestHelpers::SecondDSL.add_attribute :name
      DSLTestHelpers::SecondDSL.creates_object DSLTestHelpers::ThingOne
      DSLTestHelpers::DSL.add_object :thing, DSLTestHelpers::SecondDSL
    end

    it "defines a method for constructing an object of the real type" do
      dsl = run_dsl do
        thing do
          name 'Charlie'
        end
      end

      thing = dsl.instance_eval { @thing }
      expect(thing).to be_a DSLTestHelpers::ThingOne
      expect(thing.name).to eq 'Charlie'
    end

    it "makes the object required by default" do
      expect(DSLTestHelpers::DSL.required_attributes).to eq [:thing]
    end

    it "can accept optional attributes" do
      DSLTestHelpers::DSL.clear
      DSLTestHelpers::DSL.add_object :thing, DSLTestHelpers::SecondDSL, required: false
      expect(DSLTestHelpers::DSL.required_attributes).to eq []
    end

    context 'with no required attributes on the second DSL' do
      before do
        DSLTestHelpers::SecondDSL.clear
        DSLTestHelpers::SecondDSL.add_attribute :name, 'Jim'
        DSLTestHelpers::SecondDSL.creates_object DSLTestHelpers::ThingOne
      end

      it 'allows created objects with no block' do
        dsl = run_dsl do
          thing
        end

        thing = dsl.instance_eval { @thing }
        expect(thing).to be_a DSLTestHelpers::ThingOne
        expect(thing.name).to eq 'Jim'
      end
    end
  end

  describe "add_object_list" do
    before do
      DSLTestHelpers::SecondDSL.add_attribute :name
      DSLTestHelpers::SecondDSL.creates_object DSLTestHelpers::ThingOne
      DSLTestHelpers::DSL.add_object_list :thing, DSLTestHelpers::SecondDSL
    end

    it "defines methods for building a list of objects" do
      dsl = run_dsl do
        thing do
          name 'Jim'
        end

        thing do
          name 'Bob'
        end
      end

      things = dsl.instance_eval { @things }
      expect(things.count).to eq 2
      expect(things[0]).to be_a DSLTestHelpers::ThingOne
      expect(things[0].name).to eq 'Jim'
      expect(things[1]).to be_a DSLTestHelpers::ThingOne
      expect(things[1].name).to eq 'Bob'
    end

    it "gives an empty list as the default value" do
      dsl = run_dsl {}
      expect(dsl.instance_eval { @things }).to eq []
    end

    context 'with no required attributes on the second DSL' do
      before do
        DSLTestHelpers::SecondDSL.clear
        DSLTestHelpers::SecondDSL.add_attribute :name, 'Jim'
        DSLTestHelpers::SecondDSL.creates_object DSLTestHelpers::ThingOne
      end

      it 'allows created objects with no block' do
        dsl = run_dsl do
          thing do
            name 'Charlie'
          end
          thing
        end

        things = dsl.instance_eval { @things }
        expect(things.count).to eq 2
        expect(things[0]).to be_a DSLTestHelpers::ThingOne
        expect(things[0].name).to eq 'Charlie'
        expect(things[1]).to be_a DSLTestHelpers::ThingOne
        expect(things[1].name).to eq 'Jim'
      end
    end
  end

  describe "creates_object" do
    it "sets the copied attributes and created type" do
      DSLTestHelpers::DSL.creates_object DSLTestHelpers::ThingOne, [:name, :size]
      expect(DSLTestHelpers::DSL.created_type).to eq DSLTestHelpers::ThingOne
      expect(DSLTestHelpers::DSL.copied_attributes).to eq [:name, :size]
    end

    it "uses the full list of attributes as the copied attributes by default" do
      DSLTestHelpers::DSL.add_attribute :name
      DSLTestHelpers::DSL.add_attribute :color
      DSLTestHelpers::DSL.creates_object DSLTestHelpers::ThingOne
      expect(DSLTestHelpers::DSL.created_type).to eq DSLTestHelpers::ThingOne
      expect(DSLTestHelpers::DSL.copied_attributes).to eq [:name, :color]
    end
  end

  describe "run" do
    it "does not modify the defaults when called multiple times" do
      DSLTestHelpers::DSL.add_list :thing
      case1 = run_dsl do
        thing 'a'
        thing 'b'
      end

      case2 = run_dsl do
        thing 'c'
      end

      case3 = run_dsl {}

      expect(case1.instance_eval { @things }).to eq ['a', 'b']
      expect(case2.instance_eval { @things }).to eq ['c']
      expect(case3.instance_eval { @things }).to eq []
    end
  end

  describe 'copy_local_config' do
    before do
      DSLTestHelpers::DSL.add_attribute :name
      DSLTestHelpers::DSL.add_attribute :height
      DSLTestHelpers::DSL.add_attribute :gender
      DSLTestHelpers::DSL.add_boolean :vegan

      @dsl = DSLTestHelpers::DSL.new
      RubyYacht.configuration.local_config = {name: 'John', height: 120, vegan: false, last_name: 'Smith'}
    end

    context 'with a valid attribute' do
      it 'copies the value from the local config' do
        @dsl.copy_local_config(:name)
        expect(@dsl.instance_eval { @name }).to eq 'John'
      end
    end

    context 'with a path to the dictionary' do
      it 'copies the value from the local config' do
        RubyYacht.configuration.local_config = {people: {john: {height: 150}}}
        @dsl.copy_local_config :height, from: 'people.john'
        expect(@dsl.instance_eval { @height }).to eq 150
      end
    end

    context 'with a false boolean attribute' do
      it 'leaves the value false' do
        @dsl.copy_local_config :vegan
        expect(@dsl.instance_eval { @vegan }).to be_falsey
      end
    end

    context 'with a true boolean attribute' do
      it 'sets the value to true' do
        RubyYacht.configuration.local_config[:vegan] = true
        @dsl.copy_local_config :vegan
        expect(@dsl.instance_eval { @vegan }).to be_truthy
      end
    end

    context 'with an attribute that is not defined on the DSL' do
      it 'raises an exception' do
        expect { @dsl.copy_local_config :last_name }.to raise_exception 'Undefined field in DSLTestHelpers::DSL: last_name'
      end
    end

    context 'with an attribute with no value in the config file' do
      it 'leaves the value nil' do
        @dsl.copy_local_config :gender
        expect(@dsl.instance_eval { @gender }).to be_nil
      end
    end
  end


  describe "check_required_attributes" do
    before do
      DSLTestHelpers::DSL.add_attribute :name
      DSLTestHelpers::DSL.add_attribute :size, nil, false
    end

    it "does not throw an exception when all attributes are set" do
      dsl = DSLTestHelpers::DSL.new
      dsl.run(Proc.new do
        name 'Jim'
        size 10
      end)
      expect(lambda { dsl.check_required_attributes}).not_to raise_exception
    end

    it "does not throw an exception when an optional attribute is missing" do
      dsl = DSLTestHelpers::DSL.new
      dsl.run(Proc.new do
        name 'Jim'
      end)
      expect(lambda { dsl.check_required_attributes}).not_to raise_exception
    end

    it "throws an exception when a required attribute is nil" do
      dsl = DSLTestHelpers::DSL.new
      dsl.run(Proc.new do
        size 10
      end)
      expect(lambda { dsl.check_required_attributes}).to raise_exception 'Missing required attribute name for DSLTestHelpers::DSL'
    end

    it "throws an exception when a required attribute is an empty string" do
      dsl = DSLTestHelpers::DSL.new
      dsl.run(Proc.new do
        name ''
        size 10
      end)
      expect(lambda { dsl.check_required_attributes}).to raise_exception 'Missing required attribute name for DSLTestHelpers::DSL'
    end
  end
end