require 'spec_helper'

describe RubyYacht::Hook do
  describe "dsl" do
    let(:dsl) { RubyYacht::Hook::DSL.new(:before, :startup) }
    let(:hook) { dsl.run(@config_block).create_object }

    before do
      RubyYacht.configuration.clear

      RubyYacht.configure do
        server_type :rails do
          container_type :app
          baseline_image 'ubuntu'
        end

        server_type :mysql do
          container_type :database
          baseline_image 'ubuntu'
        end

        server_type :nginx do
          container_type :web
          baseline_image 'ubuntu'
        end
      end

      @config_block = Proc.new do
        container_type :app
        app_server_type :rails
        database_server_type :mysql
        web_server_type :nginx
      end
    end

    it "can create a hook for running a command" do
      expect(hook.event_time).to eq :before
      expect(hook.event_type).to eq :startup
      expect(hook.app_server_type).to eq :rails
      expect(hook.database_server_type).to eq :mysql
      expect(hook.web_server_type).to eq :nginx
      expect(hook.behaviors).to eq []
    end
    
    context "with no container type" do
      before do
        @config_block = Proc.new do
          app_server_type :rails
          database_server_type :mysql
          web_server_type :nginx
        end
      end
      
      it "defaults to all" do
        expect { hook }.to raise_exception 'Missing required attribute container_type for RubyYacht::Hook::DSL'
      end
    end

    context "with no app type" do
      before do
        @config_block = Proc.new do
          container_type :app
          database_server_type :mysql
          web_server_type :nginx
        end
      end
      
      it "defaults to all" do
        expect(hook.app_server_type).to eq :all
      end
    end

    context "with an app type that is not defined" do
      before do
        @config_block = Proc.new do
          container_type :app
          app_server_type :invalid
          database_server_type :mysql
          web_server_type :nginx
        end
      end
      
      it "raises an exception" do
        expect { hook }.to raise_exception 'RubyYacht::Hook::DSL has invalid app server type `invalid`'
      end
    end

    context 'with an app type that is really a database type' do
      before do
        @config_block = Proc.new do
          container_type :app
          app_server_type :mysql
          database_server_type :mysql
          web_server_type :nginx
        end
      end
      
      it "raises an exception" do
        expect { hook }.to raise_exception 'RubyYacht::Hook::DSL has invalid app server type `mysql`'
      end
    end

    context "with no database server type" do
      before do
        @config_block = Proc.new do
          container_type :app
          app_server_type :rails
          web_server_type :nginx
        end
      end
      
      it "defaults to all" do
        expect(hook.database_server_type).to eq :all
      end
    end

    context 'with a database server type that is not defined' do
      before do
        @config_block = Proc.new do
          container_type :app
          app_server_type :rails
          database_server_type :invalid
          web_server_type :nginx
        end
      end
      
      it "raises an exception" do
        expect { hook }.to raise_exception 'RubyYacht::Hook::DSL has invalid database server type `invalid`'
      end
    end

    context 'with a database server type that is really an app type' do
      before do
        @config_block = Proc.new do
          container_type :app
          app_server_type :rails
          database_server_type :rails
          web_server_type :nginx
        end
      end
      
      it "raises an exception" do
        expect { hook }.to raise_exception 'RubyYacht::Hook::DSL has invalid database server type `rails`'
      end
    end

    context "with no web server type" do
      before do
        @config_block = Proc.new do
          container_type :app
          app_server_type :rails
          database_server_type :mysql
        end
      end
      
      it "defaults to all" do
        expect(hook.web_server_type).to eq :all
      end
    end

    context 'with a web server type that is not defined' do
      before do
        @config_block = Proc.new do
          container_type :app
          app_server_type :rails
          database_server_type :mysql
          web_server_type :invalid
        end
      end
      
      it "raises an exception" do
        expect { hook }.to raise_exception 'RubyYacht::Hook::DSL has invalid web server type `invalid`'
      end
    end

    context 'with a web server type that is really an app type' do
      before do
        @config_block = Proc.new do
          container_type :app
          app_server_type :rails
          database_server_type :mysql
          web_server_type :rails
        end
      end
      
      it "raises an exception" do
        expect { hook }.to raise_exception 'RubyYacht::Hook::DSL has invalid web server type `rails`'
      end
    end

    describe 'behaviors' do
      describe 'command' do
        before do
          @config_block = Proc.new do
            container_type :app
            app_server_type :rails
            database_server_type :mysql
            command 'curl google.com'
          end
        end

        it 'adds a behavior for running the command' do
          expect(hook.behaviors.count).to eq 1
          expect(hook.behaviors[0].dockerfile_command).to eq 'RUN curl google.com'
        end
      end

      describe "run_script" do
        context 'with a script folder' do
          before do
            @config_block = Proc.new do
              container_type :app
              app_server_type :rails
              database_server_type :mysql
              script_folder './foo'
              run_script :ruby, 'bar.rb'
            end
          end

          it 'adds behaviors for copying the file, making it executable, and running it' do
            expect(hook.behaviors.count).to eq 2
            expect(hook.behaviors[0].dockerfile_command).to eq 'COPY bar.rb /var/docker/'
            expect(hook.behaviors[0].file_path).to eq './foo/bar.rb'
            expect(hook.behaviors[1].dockerfile_command).to eq 'RUN ruby /var/docker/bar.rb'
          end
        end

        context "with no script folder" do
          before do
            @config_block = Proc.new do
              container_type :app
              app_server_type :rails
              database_server_type :mysql
              run_script :bash, 'bar.bash'
            end
          end

          it "copies the file from the working directory" do
            expect(hook.behaviors.count).to eq 2
            expect(hook.behaviors[0].file_path).to eq './bar.bash'
            expect(hook.behaviors[1].dockerfile_command).to eq 'RUN bash /var/docker/bar.bash'
          end
        end
      end

      describe 'copy_file' do
        before do
          @config_block = Proc.new do
            container_type :app
            app_server_type :rails
            database_server_type :mysql
            script_folder './foo'
            copy_file 'test.txt'
          end
        end

        it "adds a behavior for copying the file" do
          expect(hook.behaviors.count).to eq 1
          expect(hook.behaviors[0].dockerfile_command).to eq 'COPY test.txt /var/docker/'
          expect(hook.behaviors[0].file_path).to eq './foo/test.txt'
        end
      end

      describe 'set_environment_variable' do
        context 'with a literal value' do
          before do
            @config_block = Proc.new do
              container_type :app
              app_server_type :rails
              database_server_type :mysql
              set_environment_variable 'RAILS_ENV', 'development'
            end
          end

          it 'adds a behavior for setting the environment variable' do
            expect(hook.behaviors.count).to eq 1
            expect(hook.behaviors[0].dockerfile_command).to eq 'ENV RAILS_ENV development'
          end
        end

        context 'with a proc value' do
          before do
            @config_block = Proc.new do
              container_type :app
              app_server_type :rails
              database_server_type :mysql
              set_environment_variable('RAILS_ENV') { 'production' }
            end
          end

          it 'adds a behavior for setting the environment variable' do
            expect(hook.behaviors.count).to eq 1
            expect(hook.behaviors[0].dockerfile_command).to eq 'ENV RAILS_ENV production'
          end
        end
      end
    end
  end

  describe "behaviors" do
    describe 'default behavior' do
      let(:behavior) { RubyYacht::Hook::Behavior.new }
      it 'has no commands' do
        expect(behavior.dockerfile_command).to eq ''
        expect(behavior.shell_command).to eq ''
      end
    end

    describe "command behavior" do
      let(:behavior) { RubyYacht::Hook::CommandBehavior.new('curl google.com') }

      it "runs the command as its docker command" do
        expect(behavior.dockerfile_command).to eq "RUN curl google.com"
      end

      it "runs the command as its shell command" do
        expect(behavior.shell_command).to eq "curl google.com"
      end
    end

    describe "copy file behavior" do
      let(:behavior) { RubyYacht::Hook::CopyFileBehavior.new('./files/script.rb') }

      it "makes the full path accessible through the file_path" do
        expect(behavior.file_path).to eq './files/script.rb'
      end

      it "copies the file as its docker command" do
        expect(behavior.dockerfile_command).to eq "COPY script.rb /var/docker/"
      end

      it "has no shell command" do
        expect(behavior.shell_command).to eq ""
      end
    end

    describe "environment variable behavior" do
      context 'with a normal value' do
        let(:behavior) { RubyYacht::Hook::EnvironmentVariableBehavior.new('SECRET_KEY_BASE', Proc.new { 'abc'}) }

        it 'sets the environment variable as its docker command' do
          expect(behavior.dockerfile_command).to eq 'ENV SECRET_KEY_BASE abc'
        end

        it 'sets the environment variable as its shell command' do
          expect(behavior.shell_command).to eq 'export SECRET_KEY_BASE="abc"'
        end
      end

      context 'with a blank value' do
        let(:behavior) { RubyYacht::Hook::EnvironmentVariableBehavior.new('SECRET_KEY_BASE', Proc.new { '' }) }

        it 'sets the environment variable as its docker command' do
          expect(behavior.dockerfile_command).to eq 'ENV SECRET_KEY_BASE '
        end

        it 'sets the environment variable as its shell command' do
          expect(behavior.shell_command).to eq 'export SECRET_KEY_BASE=""'
        end
      end

      context 'with a nil value' do
        let(:behavior) { RubyYacht::Hook::EnvironmentVariableBehavior.new('SECRET_KEY_BASE', Proc.new { nil }) }

        it 'has a nil command' do
          expect(behavior.dockerfile_command).to be_nil
          expect(behavior.shell_command).to be_nil
        end
      end
    end
  end
end