require 'spec_helper'

describe RubyYacht::Project do
  describe "dsl" do
    let(:project) { RubyYacht::Project::DSL.new(:test_project).run(@builder).create_object }

    before do
      RubyYacht.configuration.clear
      RubyYacht.configure do
        server_type :generic do
          container_type :app
          baseline_image 'ubuntu'
        end
        server_type :sqlite do
          container_type :database
          baseline_image 'ubuntu'
        end
        server_type :apache do
          container_type :web
          baseline_image 'ubuntu'
        end
      end
    end
    
    it "can create a project with all the fields" do
      @builder = Proc.new do
        system_prefix :a
        repository "github.com"
        repository_protocol :https

        app :generic, :app1 do
          repository_name 'brownleej/test1'
        end

        app :generic, :app2 do
          repository_name 'brownleej/test2'
        end

        database :sqlite, :project1 do
          host "localhost"
          username "test"
          password "test"
          port 1234
          container_label :sqlite
        end

        dns_server do
          server '8.10.1.1'
        end

        web_server :apache, :web do
          domain 'test.com'
        end

        primary_app :app1
      end

      expect(project.name).to eq :test_project
      expect(project.system_prefix).to eq :a
      expect(project.repository).to eq "github.com"
      expect(project.repository_protocol).to eq :https
      expect(project.primary_app).to eq :app1

      expect(project.apps.map(&:name)).to eq [:app1, :app2]
      expect(project.databases.map(&:name)).to eq [:project1]
      expect(project.web_servers.map(&:name)).to eq [:web]

      expect(project.dns_server.servers).to eq ['8.10.1.1']
    end

    it "requires the system prefix" do
      @builder = Proc.new do
        repository 'github.com'
        repository_protocol :https

        app :generic, :app1 do
          repository_name 'brownleej/test1'
        end

        app :generic, :app2 do
          repository_name 'brownleej/test2'
        end

        database :sqlite, :project1 do
          host "localhost"
          username "test"
          password "test"
          port 1234
          container_label :sqlite
        end

        web_server :apache, :web do
          domain 'test.com'
        end

        dns_server do
          server '8.10.1.1'
        end

        primary_app :app1
      end

      expect { project }.to raise_exception "Missing required attribute system_prefix for RubyYacht::Project::DSL"
    end

    it "requires the repository" do
      @builder = Proc.new do
        system_prefix :a
        repository_protocol :https

        app :generic, :app1 do
          repository_name 'brownleej/test1'
        end

        app :generic, :app2 do
          repository_name 'brownleej/test2'
        end

        database :sqlite, :project1 do
          host "localhost"
          username "test"
          password "test"
          port 1234
          container_label :sqlite
        end

        web_server :apache, :web do
          domain 'test.com'
        end

        dns_server do
          server '8.10.1.1'
        end

        primary_app :app1
      end

      expect { project }.to raise_exception "Missing required attribute repository for RubyYacht::Project::DSL"
    end

    it 'defaults the repository protocol to ssh' do
      @builder = Proc.new do
        system_prefix :a
        repository 'github.com'

        app :generic, :app1 do
          repository_name 'brownleej/test1'
        end

        app :generic, :app2 do
          repository_name 'brownleej/test2'
        end

        database :sqlite, :project1 do
          host "localhost"
          username "test"
          password "test"
          port 1234
          container_label :sqlite
        end

        web_server :apache, :web do
          domain 'test.com'
        end

        dns_server do
          server '8.10.1.1'
        end

        primary_app :app1
      end

      expect(project.repository_protocol).to eq :ssh
    end

    it "does not require any apps" do
      @builder = Proc.new do
        system_prefix :a
        repository "github.com"
        repository_protocol :https

        database :sqlite, :project1 do
          host "localhost"
          username "test"
          password "test"
          port 1234
          container_label :sqlite
        end

        web_server :apache, :web do
          domain 'test.com'
        end

        dns_server do
          server '8.10.1.1'
        end

        primary_app :app1
      end

      expect(project.apps.count).to eq 0
    end
    
    it "does not require any web servers" do
      @builder = Proc.new do
        system_prefix :a
        repository "github.com"
        repository_protocol :https

        app :generic, :app1 do
          repository_name 'brownleej/test1'
        end

        app :generic, :app2 do
          repository_name 'brownleej/test2'
        end

        database :sqlite, :project1 do
          host "localhost"
          username "test"
          password "test"
          port 1234
          container_label :sqlite
        end

        dns_server do
          server '8.10.1.1'
        end

        primary_app :app1
      end

      expect(project.web_servers.count).to eq 0
    end

    it "does not require the database config" do
      @builder = Proc.new do
        system_prefix :a
        repository "github.com"
        repository_protocol :https

        app :generic, :app1 do
          repository_name 'brownleej/test1'
        end

        app :generic, :app2 do
          repository_name 'brownleej/test2'
        end

        web_server :apache, :web do
          domain 'test.com'
        end

        dns_server do
          server '8.10.1.1'
        end

        primary_app :app1
      end

      expect(project.databases.count).to eq 0
    end

    it "does not require the DNS server config" do
      @builder = Proc.new do
        system_prefix :a
        repository "github.com"
        repository_protocol :https

        app :generic, :app1 do
          repository_name 'brownleej/test1'
        end

        app :generic, :app2 do
          repository_name 'brownleej/test2'
        end

        database :sqlite, :project1 do
          host "localhost"
          username "test"
          password "test"
          port 1234
          container_label :sqlite
        end

        web_server :apache, :web do
          domain 'test.com'
        end

        primary_app :app1
      end

      expect(project.dns_server).to be_nil
    end
    
    it "does not require the primary_app field" do
      @builder = Proc.new do
        system_prefix :a
        repository "github.com"
        repository_protocol :https

        app :generic, :app1 do
          repository_name 'brownleej/test1'
        end

        app :generic, :app2 do
          repository_name 'brownleej/test2'
        end

        database :sqlite, :project1 do
          host "localhost"
          username "test"
          password "test"
          port 1234
          container_label :sqlite
        end

        web_server :apache, :web do
          domain 'test.com'
        end

        dns_server do
          server '8.10.1.1'
        end
      end

      expect(project.primary_app).to be_nil
    end
  end
end