require 'spec_helper'

describe RubyYacht::ServerType do
  let(:full_config_block) do
    Proc.new do
      container_type :app
      baseline_image 'ubuntu'
      project_attribute name: :environment, default: 'staging'
      project_attribute name: :foo

      server_attribute name: :bar, required: false

      server_default database_name: 'foo_db'
    end
  end

  before do
    RubyYacht.configuration.clear
    RubyYacht.configure do
      server_type :generic  do
        container_type :app
        baseline_image 'ubuntu'
      end
      
      server_type :sqlite do
        container_type :database
        baseline_image 'ubuntu'
      end
    end
  end
  
  describe "setting attributes on other DSL types" do
    let(:project_configuration_block) do
      Proc.new do
        system_prefix 'test-project'
        repository 'github.com'

        database :sqlite, :project1 do
          host "localhost"
          username "test"
          password "test"
          port 1234
          container_label :sqlite
        end
      end
    end

    before do
      block = full_config_block
      RubyYacht.configure do
        server_type :app_type_test do
          instance_eval(&block)
        end
      end
    end

    it "defines a method for adding an app with this type to the project" do
      block = project_configuration_block
      RubyYacht.configure do
        project :test do
          instance_eval(&block)
          app_type_test_foo 'test value'

          app_type_test_app :test do
            repository_name 'brownleej/test'
          end
        end
      end

      app = RubyYacht.configuration.projects.last.apps.first
      expect(app.name).to eq :test
      expect(app.server_type).to eq :app_type_test
    end

    it "creates attributes for the project attributes on the Project DSL" do
      block = project_configuration_block
      RubyYacht.configure do
        project :test do
          instance_eval(&block)
          app_type_test_foo 'test value'
        end
      end

      project = RubyYacht.configuration.projects.last
      expect(RubyYacht::Project::DSL.new(:test).singleton_class.all_attributes).to include :app_type_test_foo
      expect(project.app_type_test_environment).to eq 'staging'
      expect(project.app_type_test_foo).to eq 'test value'
    end

    it "creates attributes for the app attributes on apps of this type" do
      block = project_configuration_block
      RubyYacht.configure do
        project :test do
          instance_eval(&block)
          app_type_test_foo 'test value'

          app :app_type_test, :test1 do
            app_type_test_bar 'baz'
            repository_name 'brownleej/test'
          end
        end
      end
      
      expect(RubyYacht::App::DSL.new(:app_type_test, :test1).singleton_class.all_attributes).to include :app_type_test_bar

      project = RubyYacht.configuration.projects.last
      app = project.apps.last
      expect(app.app_type_test_bar).to eq 'baz'
    end

    it "does not create attributes on apps of other types" do
      raised_exception = false
      block = project_configuration_block
      RubyYacht.configure do
        project :test do
          instance_eval(&block)
          app_type_test_foo 'test value'

          app :generic, :test1 do
            begin
              app_type_test_bar 'baz'
            rescue
              raised_exception = true
            end
            repository_name 'brownleej/test'
          end
        end
      end

      project = RubyYacht.configuration.projects.last
      app = project.apps.last
      expect(raised_exception).to be_truthy
      expect(app.app_type_test_bar).to be_nil
    end

    context "with a missing required attribute" do
      it "raises an exception" do
        block = project_configuration_block
        expect(lambda do
          RubyYacht.configure do
            project :test do
              instance_eval(&block)
            end
          end
        end).to raise_exception "Missing required attribute app_type_test_foo for RubyYacht::Project::DSL"
      end
    end

    context "with a default value for an existing server attribute" do
      before do
        block = project_configuration_block
        RubyYacht.configure do
          server_type :app_type_test2 do
            container_type :app
            baseline_image 'ubuntu'
          end
        end

        RubyYacht.configure do
          project :test do
            instance_eval(&block)
            app_type_test_foo 'test value'

            app_type_test_app :app1 do
              repository_name 'brownleej/test'
            end

            app_type_test_app :app2 do
              repository_name 'brownleej/test'
              database_name 'testdb'
            end

            app_type_test2_app :app3 do
              repository_name 'brownleej/test'
            end
          end
        end
      end

      it "sets those default values" do
        app = RubyYacht.configuration.projects.first.apps[0]
        expect(app.database_name).to eq 'foo_db'
      end

      it "does not override explicit values" do
        app = RubyYacht.configuration.projects.first.apps[1]
        expect(app.database_name).to eq 'testdb'
      end

      it "does not set a default value on servers of other types" do
        app = RubyYacht.configuration.projects.first.apps[2]
        expect(app.database_name).to be_nil
      end
    end
  end

  describe "dsl" do
    let(:dsl) { RubyYacht::ServerType::DSL.new(:app_type_test) }
    let(:server_type) { dsl.run(@config_block).create_object }

    before do
      @config_block = full_config_block
    end

    it "sets the information on the app type" do
      expect(server_type.name).to eq :app_type_test
      expect(server_type.container_type).to eq :app
      expect(server_type.baseline_image).to eq 'ubuntu'
      
      expect(server_type.project_attributes.count).to eq 2
      expect(server_type.project_attributes[0]).to eq(name: :environment, default: 'staging')
      expect(server_type.project_attributes[1]).to eq(name: :foo)
      
      expect(server_type.server_attributes.count).to eq 1
      expect(server_type.server_attributes[0]).to eq(name: :bar, required: false)

      @server = Object.new
      @server.define_singleton_method :secret_key do
        'abc1234'
      end
    end

    context "with no baseline image" do
      it "raises an exception" do
        @config_block = Proc.new do
          container_type :app
        end

        expect { server_type }.to raise_exception "Missing required attribute baseline_image for RubyYacht::ServerType::DSL"
      end
    end

    context "with no container type" do
      it "raises an exception" do
        @config_block = Proc.new do
          baseline_image 'ubuntu'
        end
        
        expect { server_type }.to raise_exception "Missing required attribute container_type for RubyYacht::ServerType::DSL"
      end
    end
  end
end