require 'spec_helper'

describe RubyYacht::WebServer do
  describe "DSL" do
    let(:name) { :test_app }
    let(:server) { RubyYacht::WebServer::DSL.new(:test, name).run(@builder).create_object }

    before do
      RubyYacht.configuration.clear
      RubyYacht.configure do
        server_type :test do
          container_type :web
          baseline_image 'ubuntu'
        end
      end
    end

    it 'can create a web server with all attributes' do
      @builder = Proc.new do
        domain 'test.com'
        port 8080
      end

      expect(server.name).to eq :test_app
      expect(server.domain).to eq 'test.com'
      expect(server.port).to eq 8080
    end

    it 'defaults the name to web' do
      @builder = Proc.new do
        domain 'test.com'
        port 8080
      end
      server = RubyYacht::WebServer::DSL.new(:test).run(@builder).create_object

      expect(server.name).to eq :web
    end

    it 'requires the domain' do
      @builder = Proc.new do
        port 8080
      end

      expect { server }.to raise_exception 'Missing required attribute domain for RubyYacht::WebServer::DSL'
    end

    it 'defaults the port to 80' do
      @builder = Proc.new do
        domain 'test.com'
      end

      expect(server.port).to eq 80
    end

    it 'requires a defined server type' do
      @builder = Proc.new do
        domain 'test.com'
        port 8080
        server_type :invalid
      end
      expect do
        self.server
      end.to raise_exception("RubyYacht::WebServer::DSL has invalid web server type `invalid`")
    end

    it 'requires a defined server type for web servers' do
      RubyYacht.configure do
        server_type :sqlite do
          container_type :database
          baseline_image 'ubuntu'
        end
      end

      @builder = Proc.new do
        domain 'test.com'
        port 8080
        server_type :sqlite
      end
      expect do
        self.server
      end.to raise_exception("RubyYacht::WebServer::DSL has invalid web server type `sqlite`")
    end

    it 'allows creating a server with the default name in the project DSL' do
      RubyYacht.configure do
        project :project1 do
          repository 'github.com'
          system_prefix :project1

          test_web_server do
            domain 'project1.test.com'
          end
        end
      end

      server = RubyYacht.configuration.projects.first.web_servers.first
      expect(server.name).to eq :web
    end
  end

  describe 'project' do
    before do
      RubyYacht.configuration.clear
      RubyYacht.configure do
        server_type :test do
          container_type :web
          baseline_image 'ubuntu'
        end
      end
      RubyYacht.configure do
        project :project1 do
          repository 'github.com'
          system_prefix :project1

          test_web_server :server1 do
            domain 'project1.test.com'
          end

          test_web_server :server2 do
            domain 'project2.test.com'
          end
        end
        
        project :project2 do
          repository 'github.com'
          system_prefix :project2

          test_web_server :app3 do
            domain 'project3.test.com'
          end
        end
      end
    end

    it 'finds the project that contains the server' do
      server2 = RubyYacht.configuration.projects[0].web_servers[1]
      expect(server2.project.name).to eq :project1

      server3 = RubyYacht.configuration.projects[1].web_servers[0]
      expect(server3.project.name).to eq :project2
    end
  end

  describe 'container_name' do
    before do
      @project = RubyYacht::Project.new
      @project.name = :test_project
      @project.system_prefix = :tests

      @server = RubyYacht::WebServer.new
      @server.name = 'server1'
      @project.web_servers = [@server]
      RubyYacht.configuration.projects << @project
    end

    it "combines the project's prefix with the server's name" do
      expect(@server.container_name).to eq 'tests-server1'
    end
  end
end