require 'spec_helper'

describe 'normal config', :integration do
  include IntegrationHelpers

  before :all do
    run_command 'build_images'
  end
    
  after :all do
    IntegrationHelpers.clear_docker_containers
  end

  describe 'building images' do
    after do
      system 'docker rm -f apollo-database 1> /dev/null 2> /dev/null'
    end

    it "creates images for the app, dependencies, database, and web server" do
      expect(images).to eq ['apollo-database', 'apollo-mars', 'apollo-rails-app-dependencies', 'apollo-saturn', 'apollo-web']
    end 
    
    it "populates the tables in the database image" do
      start_database
      
      results = run_command_in_docker :mars, 'mysql -uapollo -ptestpass -hapollo-database -e "use apollo; show tables"'
      results = results.split("\n").collect { |line| line.delete('|').strip }.sort
      results.shift
      expect(results).to eq %w(clients posts schema_migrations users)
    end
    
    it "populates the seed data in the database image" do
      start_database
      
      results = run_command_in_docker :mars, 'mysql -uapollo -ptestpass -hapollo-database -e "select count(*) FROM apollo.users"'
      results = results.split("\n").collect { |line| line.delete('|').strip }
      results.shift
      expect(results).to eq ['1']
      
      results = run_command_in_docker :mars, 'mysql -uapollo -ptestpass -hapollo-database -e "select name FROM apollo.clients"'
      results = results.split("\n").collect { |line| line.delete('|').strip }
      results.shift
      expect(results).to eq ['Test Client']
    end

    it "creates a test database" do
      start_database

      results = run_command_in_docker :mars, 'mysql -uapollo -ptestpass -hapollo-database apollo_test 2>&1'
      expect(results).not_to include 'ERROR'
    end
    
    it "installs the gems on the app dependency image" do
      results = run_command_in_docker 'rails-app-dependencies', 'gem list'
      expect(results).to include 'mysql2'
      expect(results).to include 'sass-rails'
      expect(results).to include 'byebug'
      expect(results).to include 'factory_girl'
    end
    
    it "leaves all the apps checked out on the app dependency image" do
      results = run_command_in_docker 'rails-app-dependencies', 'ls /var/code'
      folders = results.split("\n")
      expect(folders).to include 'mars'
      expect(folders).to include 'saturn'
    end
    
    it "installs the gems on the app images" do
      results = run_command_in_docker :mars, 'gem list'
      expect(results).to include 'mysql2'
      expect(results).not_to include 'sass-rails'
      expect(results).to include 'byebug'
      expect(results).to include 'factory_girl'

      results = run_command_in_docker :saturn, 'gem list'
      expect(results).to include 'mysql2'
      expect(results).to include 'sass-rails'
      expect(results).not_to include 'byebug'
      expect(results).not_to include 'factory_girl'
    end

    it 'installs mysql on the app images' do
      results = run_command_in_docker :mars, 'which mysql'
      expect(results).to eq "/usr/bin/mysql\n"
    end
    
    it "checks out just the single app on the app image" do
      results = run_command_in_docker :mars, 'ls /var/code'
      folders = results.split("\n")
      expect(folders).to include 'app'
      expect(folders).to include 'db'
    end
    
    it "installs the scripts in the /var/docker folder in the app image" do
      results = run_command_in_docker :mars, 'ls /var/docker'
      files = results.split("\n")
      expect(files).to include 'checkout.bash'
      expect(files).to include 'before_startup.bash'
      expect(files).to include 'startup.bash'
      expect(files).to include 'update_rails_config.rb'
      expect(files).to include 'prepare_rails_for_launch.rb'
    end

    it 'leaves the RAILS_ENV empty' do
      results = run_command_in_docker :mars, 'env'
      expect(results).not_to include 'RAILS_ENV'
    end
  end

  describe 'running containers' do
    before :all do
      run_command 'run_containers'
      
      wait_on_command_success 'curl localhost'
      wait_on_command_result 'curl -I -H Host:mars.apollo.johnbrownlee.com localhost', contains: '200 OK'
      wait_on_command_result 'curl -I -H Host:saturn.apollo.johnbrownlee.com localhost', contains: '200 OK'
    end
    
    it "starts all the containers" do
      expect(containers).to eq %w(apollo-database apollo-mars apollo-saturn apollo-web)
    end
    
    it "makes the app servers accessible from the web server" do
      results = run_command_in_docker :web, 'bash -c "apt-get install -y curl 2>&1; curl apollo-mars:8080/users 2>&1"'
      expect(results).to include 'test_user@mars.apollo.johnbrownlee.com'
    end
    
    it "makes the web server accessible from the host machine" do
      results = `curl -H Host:apollo.johnbrownlee.com localhost 2>&1`
      expect(results).to include '<a href="http://mars.apollo.johnbrownlee.com/">'
    end
    
    it "makes the app servers accessible from the host machine through the web server" do
      results = `curl -H Host:mars.apollo.johnbrownlee.com localhost/users 2>&1`
      expect(results).to include 'test_user@mars.apollo.johnbrownlee.com'

      results = `curl -H Host:saturn.apollo.johnbrownlee.com localhost 2>&1`
      expect(results).to include 'Welcome to Saturn'
    end
    
    it "does not generate static assets" do
      results = `docker exec -it apollo-mars ls /var/code/public`
      expect(results).not_to include 'assets'
    end

    it 'starts the server in development mode' do
      results = `curl -H Host:mars.apollo.johnbrownlee.com localhost/bad/path 2>&1`
      expect(results).to include '<title>Action Controller: Exception caught</title>'
    end
  end

  describe 'build command' do
    before :all do
      run_command 'build'
      wait_on_command_success 'curl apollo.johnbrownlee.com'
    end

    after :all do
      restore_hosts_file
    end

    it "allows accessing the containers through their hostnames" do
      results = `curl apollo.johnbrownlee.com 2>&1`
      expect(results).to include '<a href="http://mars.apollo.johnbrownlee.com/">'
    end
  end
  
  describe 'checking out new branches' do
    before :all do
      run_command 'run_containers'
      wait_on_command_result 'curl -I -H Host:mars.apollo.johnbrownlee.com localhost', contains: '200 OK'
    end

    context 'with a valid command' do
      before :all do
        @result = run_command 'checkout mars checkout-test-branch'
        wait_on_command_result 'curl -H Host:mars.apollo.johnbrownlee.com localhost/posts -d ""', contains: '"tag":null'
      end

      it 'returns true' do
        expect(@result).to be_truthy
      end

      it "updates the code from the server" do
        results = `docker exec apollo-mars ls app/controllers`.split("\n")
        expect(results).to include 'users_controller.rb'
        expect(results).to include 'posts_controller.rb'
      end

      it "runs new database migrations" do
        results = run_command_in_docker(:mars, 'mysql -uapollo -ptestpass -hapollo-database -e "show fields from apollo.posts"').split("\n")
        expect(results).to include "tag\tvarchar(255)\tYES\t\tNULL\t"
      end

      it "starts the server with the new code base" do
        results = `docker exec apollo-mars curl localhost:8080/posts -d  "user_id=1&title=First%20Post&contents=Hello,%20World&tag=party" 2>&1`
        expect(results).to include '"tag":"party"'
      end
    end

    context 'with no branch' do
      it 'returns false' do
        result = run_command 'checkout mars'
        expect(result).to be_falsey
      end
    end

    context 'with no app name' do
      it 'returns false' do
        result = run_command 'checkout'
        expect(result).to be_falsey
      end
    end
  end

  describe 'starting and stopping services' do
    before :all do
      run_command 'run_containers'
    end

    context "with the start command" do
      before :all do
        IntegrationHelpers.image_names.each do |image|
          system "docker stop #{image} 1> /dev/null 2> /dev/null"
        end

        @result = run_command 'services start'
      end

      it "returns a success" do
        expect(@result).to be_truthy
      end

      it "starts the containers" do
        results = `docker ps | grep apollo`.split("\n")
        results = results.collect { |line| line.split(/\s+/).last }
        results.sort!

        expect(results).to eq %w(apollo-database apollo-mars apollo-saturn apollo-web)
      end
    end

    context "with the stop command" do
      before :all do
        IntegrationHelpers.image_names.each do |image|
          system "docker start #{image} 1> /dev/null 2> /dev/null"
        end
        
        @result = run_command 'services stop'
      end

      it "returns a success" do
        expect(@result).to be_truthy
      end

      it "stops the containers" do
        results = `docker ps | grep apollo`
        expect(results).to eq ''

        results = `docker ps -a | grep apollo`.split("\n")
        results = results.collect { |line| line.split(/\s+/).last }
        results.sort!

        expect(results).to eq %w(apollo-database apollo-mars apollo-saturn apollo-web)
      end
    end

    context "with an invalid command" do
      before :all do
        @result = run_command 'services crash'
      end

      it "returns a failure" do
        expect(@result).to be_falsey
      end
    end

    context "with no command" do
      before :all do
        @result = run_command 'services'
      end

      it "returns a failure" do
        expect(@result).to be_falsey
      end
    end
  end

  describe 'shell' do
    before :all do
      run_command 'run_containers'
    end

    context 'with a normal command' do
      it "returns the results of running that command" do
        results = run_command 'shell mars ls', capture_output: true
        results = results.split(/\s+/)
        expect(results).to include "app"
        expect(results).to include "db"
      end
    end

    context 'with no app name' do
      it 'returns a failure' do
        results = run_command 'shell'
        expect(results).to be_falsey
      end
    end
  end

  describe 'update_hosts' do
    before :all do
      run_command 'update_hosts'
    end

    after :all do
      restore_hosts_file
    end

    it "puts the domains for the apps in the hosts file" do
      results = File.read('/etc/hosts').split("\n")
      expect(results).to include '127.0.0.1    apollo.johnbrownlee.com'
      expect(results).to include '127.0.0.1    mars.apollo.johnbrownlee.com'
      expect(results).to include '127.0.0.1    saturn.apollo.johnbrownlee.com'
    end

    it "puts an old copy in the backup" do
      file = Dir["/etc/hosts.*.backup"].last
      expect(file).not_to be_nil
      results = File.read(file).split("\n")
      expect(results.count).to be > 0
      expect(results).not_to include 'apollo.johnbrownlee.com'
      expect(results).not_to include 'mars.apollo.johnbrownlee.com'
      expect(results).not_to include 'saturn.apollo.johnbrownlee.com'
    end
  end

  describe 'imploding' do
    before :all do
      run_command 'run_containers'
      run_command 'implode'
    end

    it 'removes all the images and containers' do
      containers = `docker ps -a | grep apollo`
      expect(containers).to eq ''

      images = `docker images -a | grep apollo`
      expect(images).to eq ''
    end
  end
end