require 'spec_helper'

RSpec.describe 'project with production environment', :integration do
  include IntegrationHelpers

  before :all do
    run_command 'build_images', environment: { USE_PRODUCTION_ENVIRONMENT: 1 }
  end

  after :all do
    IntegrationHelpers.clear_docker_containers
  end

  describe 'building images' do
    it "does not install the non-production gems" do
      results = run_command_in_docker 'rails-app-dependencies', 'gem list'
      expect(results).to include 'mysql2'
      expect(results).not_to include 'byebug'
      expect(results).not_to include 'factory_girl'

      results = run_command_in_docker :mars, 'gem list'
      expect(results).to include 'mysql2'
      expect(results).not_to include 'byebug'
      expect(results).not_to include 'factory_girl'
    end

    it "does not create a test database" do
      start_database
      results = run_command_in_docker :mars, 'mysql -uapollo -ptestpass -hapollo-database apollo_test 2>&1'
      expect(results).to include 'ERROR'
    end

    it 'sets the RAILS_ENV variable' do
      results = run_command_in_docker :mars, 'env'
      expect(results).to include "RAILS_ENV=production"
    end
  end

  describe 'running containers' do
    before :all do
      run_command 'run_containers', environment: { USE_PRODUCTION_ENVIRONMENT: 1 }
      wait_on_command_result 'curl -I -H Host:mars.apollo.johnbrownlee.com localhost/users', contains: '200 OK'
    end
    
    it "starts all the containers" do
      expect(containers).to eq %w(apollo-database apollo-mars apollo-saturn apollo-web)
    end
    
    it "generates static assets" do
      results = `docker exec -it apollo-mars ls /var/code/public/assets`
      expect(results).to include 'application-89720f5a46589c09bfd16b3e1c7c509bba7d6a3c3dd4f3a8e13e0d05a0fbf9c1.js'
    end

    it 'starts the server in production mode' do
      results = `curl -H Host:mars.apollo.johnbrownlee.com localhost/bad/path 2>&1`
      expect(results).to include "<title>The page you were looking for doesn't exist (404)</title>"
    end
  end

  describe 'checking out new branches' do
    before :all do
      run_command 'run_containers', environment: { USE_PRODUCTION_ENVIRONMENT: 1 }
      run_command 'checkout mars master', environment: { USE_PRODUCTION_ENVIRONMENT: 1 }
    end
    
    it "does not install any non-production gems" do
      gems = `docker exec -it apollo-mars gem list`
      expect(gems).to include 'mysql2'
      expect(gems).not_to include 'byebug'
    end
  end
end