require 'spec_helper'

RSpec.describe 'miscellaneous scenarios', :integration do
  include IntegrationHelpers
  
  context 'with DNS server config' do
    before :all do
      run_command 'build_images', environment: { CONFIGURE_DNS: 1 }
      run_command 'run_containers', environment: { CONFIGURE_DNS: 1 }
    end
    
    after :all do
      IntegrationHelpers.clear_docker_containers
    end
    
    it "puts the custom config in the DNS server config" do
      results = `docker exec apollo-mars cat /etc/resolv.conf`
      expect(results).to include 'dns1.test.com'
    end
  end

  context 'with a primary app' do
    before :all do
      run_command 'build_images', environment: { CREATE_PRIMARY_APP: 1 }
      run_command 'run_containers', environment: { CREATE_PRIMARY_APP: 1 }
      wait_on_command_result 'curl -I -H Host:apollo.johnbrownlee.com localhost', contains: '200 OK'
      wait_on_command_result 'curl -I -H Host:saturn.apollo.johnbrownlee.com localhost', contains: '200 OK'
    end

    after :all do
      IntegrationHelpers.clear_docker_containers
    end
    
    it "starts all the containers" do
      expect(containers).to eq %w(apollo-database apollo-mars apollo-saturn apollo-web)
    end
    
    it "makes the web server accessible from the host machine" do
      results = `curl -H Host:apollo.johnbrownlee.com localhost/users 2>&1`
      expect(results).to include 'test_user@mars.apollo.johnbrownlee.com'
    end
    
    it "makes the main app accessible from the host machine through the web server" do
      results = `curl -H Host:mars.apollo.johnbrownlee.com localhost/users 2>&1`
      expect(results).to include 'test_user@mars.apollo.johnbrownlee.com'
    end

    it "makes the secondary app accessible" do
      results = `curl -H Host:saturn.apollo.johnbrownlee.com localhost 2>&1`
      expect(results).to include 'Welcome to Saturn'
    end
  end

  context 'with a non-standard database port' do
    before :all do
      run_command 'build_images', environment: { USE_NONSTANDARD_MYSQL_PORT: 1 }
      run_command 'run_containers', environment: { USE_NONSTANDARD_MYSQL_PORT: 1 }
      wait_on_command_result 'curl -I -H Host:mars.apollo.johnbrownlee.com localhost/users', contains: '200 OK'
    end

    after :all do
      IntegrationHelpers.clear_docker_containers
    end

    it 'sets up the database to listen on that port' do
      results = run_command_in_docker :mars, 'mysql -uapollo -ptestpass -hapollo-database -P3333 apollo 2>&1'
      expect(results.downcase).not_to include 'error'
    end

    it 'allows the app to connect to the database on that port' do
      results = `curl -H Host:mars.apollo.johnbrownlee.com localhost/users 2>&1`
      expect(results).to include 'test_user@mars.apollo.johnbrownlee.com'
    end
  end

  context 'with config from a local file' do
    before :all do
      run_command 'build_images', environment: { USE_LOCAL_CONFIG: 1 }
      run_command 'run_containers', environment: { USE_LOCAL_CONFIG: 1 }
      wait_on_command_result 'curl -I -H Host:mars.apollo.johnbrownlee.com localhost/users', contains: '200 OK'
    end

    after :all do
      IntegrationHelpers.clear_docker_containers
    end

    it 'allows connecting to the database with that local config' do
      results = run_command_in_docker :mars, 'mysql -ujupiter -pjupiterpass -hapollo-database apollo 2>&1'
      expect(results.downcase).not_to include 'error'
    end

    it 'sets the rails environment to the one in the file' do
      results = run_command_in_docker :mars, 'env'
      expect(results).to include 'RAILS_ENV=production'
    end
  end
end
