require 'spec_helper'

describe RubyYacht::Plugins::Rails do
  before do
    RubyYacht.configuration.clear
    described_class.load

    @project = RubyYacht::Project.new
    RubyYacht.configuration.projects << @project

    @app = RubyYacht::App.new
    @app.server_type = :rails
    @app.database_name = :test_db
    @project.apps = [@app]

    @database = RubyYacht::Database.new
    @database.name = :test_db
    @project.databases = [@database]

    @project.web_servers = []
  end
  
  def configure_project(options = {})
    RubyYacht.configuration.clear
    RubyYacht::Plugins::Rails.load

    RubyYacht.configure do
      project :test_project do
        system_prefix :test_project
        repository 'github.com'
        rails_secret_key_base(options[:secret_key_base]) if options[:secret_key_base]
        rails_environment(options[:environment]) if options[:environment]
        rails_excluded_gem_groups(options[:excluded_gem_groups]) if options[:excluded_gem_groups]
      end
    end
  end

  it "adds rails to the app type list" do
    expect(RubyYacht.configuration.server_types.map(&:name)).to include :rails
  end

  it "adds an attribute for the rails environment" do
    configure_project(secret_key_base: 'abc', environment: 'production')
    project = RubyYacht.configuration.projects.last
    expect(project.rails_environment).to eq 'production'
    
    configure_project(secret_key_base: 'abc')
    project = RubyYacht.configuration.projects.last
    expect(project.rails_environment).to eq 'development'
  end

  it "adds an attribute for the rails secret key" do
    configure_project(secret_key_base: 'abc')
    project = RubyYacht.configuration.projects.last
    expect(project.rails_secret_key_base).to eq 'abc'
  end
  
  it "adds an attribute for the rails excluded gem groups" do
    configure_project(secret_key_base: 'abc', excluded_gem_groups: %w(development test))
    project = RubyYacht.configuration.projects.last
    expect(project.rails_excluded_gem_groups).to eq %w(development test)
    
    RubyYacht.configuration.clear
    configure_project(secret_key_base: 'abc')
    project = RubyYacht.configuration.projects.last
    expect(project.rails_excluded_gem_groups).to eq []
  end

  describe "building images" do
    subject { RubyYacht::Runner::BuildImages.new }
    include DockerCommandSpec

    def configure_project
      RubyYacht.configure do
        project :apollo do
          system_prefix :apollo
          repository "github.com"
          rails_secret_key_base 'abc'
          
          sqlite_database :apollo do
            host "db.test.com"
            username "apollo"
            password "test"
            port 1234
          end
          
          rails_app :mars do
            repository_name 'brownleej/mars'
            database_name :apollo
          end
          
          rails_app :saturn do
            repository_name 'brownleej/saturn'
            database_name :apollo
          end

          nginx_web_server do
            domain 'apollo.test.com'
          end 
        end
      end
    end

    before do
      RubyYacht.configuration.clear
      described_class.load

      RubyYacht.configure do
        server_type :sqlite do
          container_type :database
          baseline_image 'ubuntu'
        end
        server_type :nginx do
          container_type :web
          baseline_image 'ubuntu'
        end
      end

      configure_project
    end
    
    it "builds all the images for the project" do
      docker.fail_unexpected_commands!
      docker.expect "network create apollo 2> /dev/null"
      docker.expect "build -t apollo-rails-app-dependencies tmp"
      docker.expect "build -t apollo-mars tmp"
      docker.expect "build -t apollo-saturn tmp"
      docker.expect "build -t apollo-web tmp"
      subject.run
    end
    
    it "generates the dockerfile for the app dependencies" do
      docker.expect("build -t apollo-rails-app-dependencies tmp") do
        files = Dir[File.join("tmp", "*")].sort
        expect(files).to eq ['tmp/Dockerfile', 'tmp/Dockerfile.erb', 'tmp/id_rsa', 'tmp/install_gems.rb']
        dockerfile = File.read(File.join('tmp', 'Dockerfile'))
        expected = File.read(File.join('spec', 'fixtures', 'app-dependencies-dockerfile-rails'))
        expect(dockerfile).to eq expected
      end
      subject.run
    end
    
    it "generates the dockerfiles for the apps" do
      docker.expect("build -t apollo-mars tmp") do
        files = Dir[File.join("tmp", "*")].sort
        expect(files).to eq ['tmp/Dockerfile', 'tmp/Dockerfile.erb', 'tmp/before_startup.bash', 'tmp/before_startup.bash.erb', 'tmp/build_new_app.rb', 'tmp/checkout.bash', 'tmp/prepare_rails_for_launch.rb', 'tmp/startup.bash', 'tmp/startup.bash.erb', 'tmp/update_rails_config.rb']
        dockerfile = File.read(File.join('tmp', 'Dockerfile'))
        expected = File.read(File.join('spec', 'fixtures', 'mars-dockerfile-rails'))
        expect(dockerfile).to eq expected
      end
      subject.run
    end

    it "builds a startup file for the apps" do
      docker.expect("build -t apollo-mars tmp") do
        startup = File.read(File.join('tmp', 'startup.bash'))
        expected = File.read(File.join('spec', 'fixtures', 'mars-startup-rails'))
        expect(startup).to eq expected
      end
      subject.run
    end

    it "builds a before-startup file for the apps" do
      docker.expect("build -t apollo-mars tmp") do
        startup = File.read(File.join('tmp', 'before_startup.bash'))
        expected = File.read(File.join('spec', 'fixtures', 'mars-before-startup-rails'))
        expect(startup).to eq expected
      end
      subject.run
    end
    
    it "generates the dockerfiles for the web server" do
      docker.expect "build -t apollo-web tmp" do
        files = Dir[File.join("tmp", "*")].sort
        expect(files).to eq ['tmp/Dockerfile', 'tmp/Dockerfile.erb']
        dockerfile = File.read(File.join('tmp', 'Dockerfile'))
        expected = File.read(File.join('spec', 'fixtures', 'web-dockerfile'))
        expect(dockerfile).to eq expected
      end
      subject.run
    end
    
    context "with a local database" do
      before do
        RubyYacht.configuration.projects.first.databases.first.host = 'localhost'
      end
      
      it "generates the dockerfiles for the database server" do
        docker.expect "build -t apollo-database tmp" do
          files = Dir[File.join("tmp", "*")].sort
          expect(files).to eq ['tmp/Dockerfile', 'tmp/Dockerfile.erb', 'tmp/checkout.bash', 'tmp/startup.bash', 'tmp/startup.bash.erb']
          dockerfile = File.read(File.join('tmp', 'Dockerfile'))
          expected = File.read(File.join('spec', 'fixtures', 'database-dockerfile-rails'))
          expect(dockerfile).to eq expected
        end
        subject.run
      end
    end
    
    context "with a production app" do
      before do
        RubyYacht.configuration.projects.first.rails_environment = 'production'
      end
      
      it "generates the dockerfiles for the app server" do
        docker.expect("build -t apollo-rails-app-dependencies tmp") do
          files = Dir[File.join("tmp", "*")].sort
          expect(files).to eq ['tmp/Dockerfile', 'tmp/Dockerfile.erb', 'tmp/id_rsa', 'tmp/install_gems.rb']
          dockerfile = File.read(File.join('tmp', 'Dockerfile'))
          expected = File.read(File.join('spec', 'fixtures', 'app-dependencies-dockerfile-rails-production'))
          expect(dockerfile).to eq expected
        end
        subject.run
      end
    end
    
    context "with a local MySQL database" do
      before do
        RubyYacht.configuration.clear
        RubyYacht::Plugins::MySQL.load
        described_class.load

        RubyYacht.configure do
          server_type :sqlite do
            container_type :database
            baseline_image 'ubuntu'
          end
          server_type :nginx do
            container_type :web
            baseline_image 'ubuntu'
          end
        end

        configure_project
        RubyYacht.configuration.projects.first.databases.first.host = 'localhost'
        RubyYacht.configuration.projects.first.databases.first.server_type = :mysql
      end
      
      it "generates the dockerfiles for the database server" do
        docker.expect "build -t apollo-database tmp" do
          files = Dir[File.join("tmp", "*")].sort
          expect(files).to eq ['tmp/Dockerfile', 'tmp/Dockerfile.erb', 'tmp/checkout.bash', 'tmp/load_seeds.rb', 'tmp/setup.bash', 'tmp/startup.bash', 'tmp/startup.bash.erb']
          dockerfile = File.read(File.join('tmp', 'Dockerfile'))
          expected = File.read(File.join('spec', 'fixtures', 'database-dockerfile-rails-mysql'))
          expect(dockerfile).to eq expected
        end
        subject.run
      end
    end
    
    context "with no repository" do
      before do
        RubyYacht.configuration.projects.first.apps.first.repository_name = nil
        RubyYacht.configuration.projects.first.databases.first.host = 'localhost'
      end
      
      it "generates the dockerfiles for the app server" do
        docker.expect "build -t apollo-mars tmp" do
          files = Dir[File.join("tmp", "*")].sort
          expect(files).to eq ['tmp/Dockerfile', 'tmp/Dockerfile.erb', 'tmp/before_startup.bash', 'tmp/before_startup.bash.erb', 'tmp/build_new_app.rb', 'tmp/checkout.bash', 'tmp/prepare_rails_for_launch.rb', 'tmp/startup.bash', 'tmp/startup.bash.erb', 'tmp/update_rails_config.rb']
          file = File.read(File.join('tmp', 'Dockerfile'))
          expected = File.read(File.join('spec', 'fixtures', 'mars-dockerfile-rails-with-no-repository'))
          expect(file).to eq expected
        end
        subject.run
      end
      
      it "generates the startup script for the app server" do
        docker.expect "build -t apollo-mars tmp" do
          file = File.read(File.join('tmp', 'startup.bash'))
          expected = File.read(File.join('spec', 'fixtures', 'mars-startup-rails-with-no-repository'))
          expect(file).to eq expected
        end
        subject.run
      end

      it "generates a dockerfile for the database server" do
        docker.expect "build -t apollo-database tmp" do
          file = File.read(File.join('tmp', 'Dockerfile'))
          expected = File.read(File.join('spec', 'fixtures', 'database-dockerfile-rails-with-no-repository'))
          expect(file).to eq expected
        end
        subject.run
      end
    end
  end
end