require 'spec_helper'

describe RubyYacht::Runner::BuildImages do
  include DockerCommandSpec
  
  describe "command information" do
    specify { expect(described_class.command).to eq 'build_images' }
    specify { expect(described_class.description).to eq 'Build images for your environment' }
  end
  
  describe "running" do
    it "builds all the images for the project" do
      docker.fail_unexpected_commands!
      docker.expect "network create apollo 2> /dev/null"
      docker.expect "build -t apollo-generic-app-dependencies tmp"
      docker.expect "build -t apollo-debian-app-dependencies tmp"
      docker.expect "build -t apollo-mars tmp"
      docker.expect "build -t apollo-saturn tmp"
      docker.expect "build -t apollo-nginx tmp"
      subject.run
    end
    
    it "generates the dockerfile for the app dependencies" do
      docker.expect("build -t apollo-generic-app-dependencies tmp") do
        files = Dir[File.join("tmp", "*")].sort
        expect(files).to eq ['tmp/Dockerfile', 'tmp/Dockerfile.erb', 'tmp/id_rsa']
        dockerfile = File.read(File.join('tmp', 'Dockerfile'))
        expected = File.read(File.join('spec', 'fixtures', 'app-dependencies-dockerfile-generic'))
        expect(dockerfile).to eq expected
      end
      subject.run
    end
    
    it "generates the dockerfiles for the apps" do
      docker.expect("build -t apollo-mars tmp") do
        files = Dir[File.join("tmp", "*")].sort
        expect(files).to eq ['tmp/Dockerfile', 'tmp/Dockerfile.erb', 'tmp/before_startup.bash', 'tmp/before_startup.bash.erb', 'tmp/checkout.bash', 'tmp/startup.bash', 'tmp/startup.bash.erb']
        dockerfile = File.read(File.join('tmp', 'Dockerfile'))
        expected = File.read(File.join('spec', 'fixtures', 'mars-dockerfile'))
        expect(dockerfile).to eq expected
      end
      subject.run
    end

    it "builds a startup file for the apps" do
      docker.expect("build -t apollo-mars tmp") do
        startup = File.read(File.join('tmp', 'startup.bash'))
        expected = File.read(File.join('spec', 'fixtures', 'mars-startup'))
        expect(startup).to eq expected
      end
      subject.run
    end

    it "builds a before-startup file for the apps" do
      docker.expect("build -t apollo-mars tmp") do
        startup = File.read(File.join('tmp', 'before_startup.bash'))
        expected = File.read(File.join('spec', 'fixtures', 'mars-before-startup'))
        expect(startup).to eq expected
      end
      subject.run
    end
    
    it "generates the dockerfiles for the web server" do
      docker.expect "build -t apollo-nginx tmp" do
        files = Dir[File.join("tmp", "*")].sort
        expect(files).to eq ['tmp/Dockerfile', 'tmp/Dockerfile.erb']
        dockerfile = File.read(File.join('tmp', 'Dockerfile'))
        expected = File.read(File.join('spec', 'fixtures', 'web-dockerfile'))
        expect(dockerfile).to eq expected
      end
      subject.run
    end
    
    it "returns true" do
      expect(subject.run).to be_truthy
    end
    
    context "with a local database" do
      before do
        RubyYacht.configure do
          server_type :sqlite do
            container_type :database
            baseline_image 'ubuntu'
          end
        end

        database = RubyYacht::Database.new
        database.name = 'apollo'
        database.server_type = :sqlite
        database.container_label = 'mysql'
        database.username = 'apollo'
        database.password = 'test'
        database.port = 1234
        database.host = 'localhost'
        RubyYacht.configuration.projects.first.databases << database
        RubyYacht.configuration.projects.first.apps.first.database_name = 'apollo'
      end

      it "builds an app dockerfile with the database config" do
        docker.expect("build -t apollo-mars tmp") do
          files = Dir[File.join("tmp", "*")].sort
          expect(files).to eq ['tmp/Dockerfile', 'tmp/Dockerfile.erb', 'tmp/before_startup.bash', 'tmp/before_startup.bash.erb', 'tmp/checkout.bash', 'tmp/startup.bash', 'tmp/startup.bash.erb']
          dockerfile = File.read(File.join('tmp', 'Dockerfile'))
          expected = File.read(File.join('spec', 'fixtures', 'mars-dockerfile-with-local-database'))
          expect(dockerfile).to eq expected
        end
        subject.run
      end
      
      it "builds the database" do
        docker.expect("build -t apollo-mysql tmp") do
          files = Dir[File.join("tmp", "*")].sort
          expect(files).to eq ['tmp/Dockerfile', 'tmp/Dockerfile.erb', 'tmp/checkout.bash', 'tmp/startup.bash', 'tmp/startup.bash.erb']
          dockerfile = File.read(File.join('tmp', 'Dockerfile'))
          expected = File.read(File.join('spec', 'fixtures', 'database-dockerfile'))
          expect(dockerfile).to eq expected
        end
        subject.run
      end
    
      it "returns true" do
        expect(subject.run).to be_truthy
      end
      
      context "with a hook for loading the seeds" do
        let(:script_folder) { File.join(File.dirname(File.dirname(__FILE__)), 'fixtures', 'hooks') }
      
        before do
        script_folder = self.script_folder
          RubyYacht.configure do
            add_hooks script_folder: script_folder, container_type: :database do
              during(:load_database_seeds) { run_script :ruby, 'hook1.rb' }
            end
          end
        end
        
        it "copies the files for the hooks to the database" do
          docker.expect("build -t apollo-mysql tmp") do
            files = Dir[File.join("tmp", "*")].sort
            expect(files).to eq ['tmp/Dockerfile', 'tmp/Dockerfile.erb', 'tmp/checkout.bash', 'tmp/hook1.rb', 'tmp/startup.bash', 'tmp/startup.bash.erb']
          end
          subject.run
        end
        
        it "includes the scripts in the database dockerfile" do
          docker.expect("build -t apollo-mysql tmp") do
            dockerfile = File.read(File.join('tmp', 'Dockerfile'))
            expected = File.read(File.join('spec', 'fixtures', 'database-dockerfile-with-seed-hooks'))
            expect(dockerfile).to eq expected
          end
          subject.run
        end
      end
    end

    context 'with a non-local database' do
      before do
        database = RubyYacht::Database.new
        database.name = 'apollo'
        database.server_type = 'mysql'
        database.container_label = 'mysql'
        database.username = 'apollo'
        database.password = 'test'
        database.host = 'db1.test.com'
        database.port = 3306
        RubyYacht.configuration.projects.first.databases << database
        RubyYacht.configuration.projects.first.apps.first.database_name = 'apollo'
      end

      it "builds an app dockerfile with the database config" do
        docker.expect("build -t apollo-mars tmp") do
          files = Dir[File.join("tmp", "*")].sort
          expect(files).to eq ['tmp/Dockerfile', 'tmp/Dockerfile.erb', 'tmp/before_startup.bash', 'tmp/before_startup.bash.erb', 'tmp/checkout.bash', 'tmp/startup.bash', 'tmp/startup.bash.erb']
          dockerfile = File.read(File.join('tmp', 'Dockerfile'))
          expected = File.read(File.join('spec', 'fixtures', 'mars-dockerfile-with-remote-database'))
          expect(dockerfile).to eq expected
        end
        subject.run
      end
      
      it "does not build an image for the database" do
        docker.fail_unexpected_commands!
        docker.expect "network create apollo 2> /dev/null"
        docker.expect "build -t apollo-generic-app-dependencies tmp"
        docker.expect "build -t apollo-debian-app-dependencies tmp"
        docker.expect "build -t apollo-mars tmp"
        docker.expect "build -t apollo-saturn tmp"
        docker.expect "build -t apollo-nginx tmp"
        subject.run
      end
    end

    context "with an app set up as the primary app" do
      before do
        RubyYacht.configuration.projects[0].primary_app = :mars
      end

      it "includes that app as the primary in the web server dockerfile" do
        docker.expect "build -t apollo-nginx tmp" do
          files = Dir[File.join("tmp", "*")].sort
          expect(files).to eq ['tmp/Dockerfile', 'tmp/Dockerfile.erb']
          dockerfile = File.read(File.join('tmp', 'Dockerfile'))
          expected = File.read(File.join('spec', 'fixtures', 'web-dockerfile-with-primary-app'))
          expect(dockerfile).to eq expected
        end
        subject.run
      end
    end

    context "with an app with the same name as the system" do
      before do
        RubyYacht.configuration.projects[0].apps[0].name = :apollo
      end

      it "builds all the images for the project" do
        docker.fail_unexpected_commands!
        docker.expect "network create apollo 2> /dev/null"
        docker.expect "build -t apollo-generic-app-dependencies tmp"
        docker.expect "build -t apollo-debian-app-dependencies tmp"
        docker.expect "build -t apollo tmp"
        docker.expect "build -t apollo-saturn tmp"
        docker.expect "build -t apollo-nginx tmp"
        subject.run
      end

      it "includes that app as the primary in the web server dockerfile" do
        docker.expect "build -t apollo-nginx tmp" do
          files = Dir[File.join("tmp", "*")].sort
          expect(files).to eq ['tmp/Dockerfile', 'tmp/Dockerfile.erb']
          dockerfile = File.read(File.join('tmp', 'Dockerfile'))
          expected = File.read(File.join('spec', 'fixtures', 'web-dockerfile-with-eponymous-app'))
          expect(dockerfile).to eq expected
        end
        subject.run
      end
    end
    
    context "with multiple projects" do
      before do
        RubyYacht.configure_second_test_project
      end
      
      it "builds the containers for both projects" do
        docker.fail_unexpected_commands!
        docker.expect "network create apollo 2> /dev/null"
        docker.expect "build -t apollo-generic-app-dependencies tmp"
        docker.expect "build -t apollo-debian-app-dependencies tmp"
        docker.expect "build -t apollo-mars tmp"
        docker.expect "build -t apollo-saturn tmp"
        docker.expect "build -t apollo-nginx tmp"
        docker.expect "network create jupiter 2> /dev/null"
        docker.expect "build -t jupiter-debian-app-dependencies tmp"
        docker.expect "build -t jupiter-venus tmp"
        docker.expect "build -t jupiter-saturn tmp"
        docker.expect "build -t jupiter-web tmp"
        subject.run
      end
    
      it "generates a normal dockerfile for the first project's web server" do
        docker.expect "build -t apollo-nginx tmp" do
          files = Dir[File.join("tmp", "*")].sort
          expect(files).to eq ['tmp/Dockerfile', 'tmp/Dockerfile.erb']
          dockerfile = File.read(File.join('tmp', 'Dockerfile'))
          expected = File.read(File.join('spec', 'fixtures', 'web-dockerfile'))
          expect(dockerfile).to eq expected
        end
        subject.run
      end
    
      it "generates a dockerfile for the second project's web server" do
        docker.expect "build -t jupiter-web tmp" do
          files = Dir[File.join("tmp", "*")].sort
          expect(files).to eq ['tmp/Dockerfile', 'tmp/Dockerfile.erb']
          dockerfile = File.read(File.join('tmp', 'Dockerfile'))
          expected = File.read(File.join('spec', 'fixtures', 'web-dockerfile-jupiter'))
          expect(dockerfile).to eq expected
        end
        subject.run
      end
    
      it "returns true" do
        expect(subject.run).to be_truthy
      end
    end

    context "with hooks before starting an app" do
      let(:script_folder) { File.join(File.dirname(File.dirname(__FILE__)), 'fixtures', 'hooks') }
      before do
        script_folder = self.script_folder
        RubyYacht.configure do
          add_hooks script_folder: script_folder, container_type: :app do
            before(:startup) { run_script :ruby, 'hook1.rb' }
            before(:startup) { run_script :ruby, 'hook2.rb' }
          end
        end
      end

      it "copies the scripts to the tmp directory" do
        docker.expect("build -t apollo-mars tmp") do
          files = Dir[File.join('tmp', '*')]
          expect(files).to include 'tmp/hook1.rb'
          expect(files).to include 'tmp/hook2.rb'
        end
        subject.run
      end

      it "builds a dockerfile that copies the scripts" do
        docker.expect("build -t apollo-mars tmp") do
          dockerfile = File.read(File.join('tmp', 'Dockerfile'))
          expected = File.read(File.join('spec', 'fixtures', 'mars-dockerfile-with-before-startup-hooks'))
          expect(dockerfile).to eq expected
        end
        subject.run
      end

      it "builds a before-startup file that runs the scripts" do
        docker.expect("build -t apollo-mars tmp") do
          startup = File.read(File.join('tmp', 'before_startup.bash'))
          expected = File.read(File.join('spec', 'fixtures', 'mars-before-startup-with-before-startup-hooks'))
          expect(startup).to eq expected
        end
        subject.run
      end
    end

    context "with hooks after checking out an app" do
      let(:script_folder) { File.join(File.dirname(File.dirname(__FILE__)), 'fixtures', 'hooks') }
      before do
        script_folder = self.script_folder
        RubyYacht.configure do
          add_hooks script_folder: script_folder, container_type: :app do
            after(:build_checkout) { run_script :ruby, 'hook1.rb' }
          end
        end
      end

      it "copies the scripts to the tmp directory" do
        docker.expect("build -t apollo-mars tmp") do
          files = Dir[File.join('tmp', '*')]
          expect(files).to include 'tmp/hook1.rb'
        end
        subject.run
      end

      it "builds a dockerfile that copies the scripts" do
        docker.expect("build -t apollo-mars tmp") do
          dockerfile = File.read(File.join('tmp', 'Dockerfile'))
          expected = File.read(File.join('spec', 'fixtures', 'mars-dockerfile-with-after-checkout-hooks'))
          expect(dockerfile).to eq expected
        end
        subject.run
      end

      it "builds a normal before-startup" do
        docker.expect("build -t apollo-mars tmp") do
          startup = File.read(File.join('tmp', 'before_startup.bash'))
          expected = File.read(File.join('spec', 'fixtures', 'mars-before-startup'))
          expect(startup).to eq expected
        end
        subject.run
      end
    end

    context "with hooks for installing libraries for an app" do
      let(:script_folder) { File.join(File.dirname(File.dirname(__FILE__)), 'fixtures', 'hooks') }
      before do
        script_folder = self.script_folder
        RubyYacht.configure do
          add_hooks script_folder: script_folder, container_type: :app do
            during(:install_libraries) { run_script :ruby, 'hook2.rb' }
          end
        end
      end

      it "copies the scripts to the tmp directory" do
        docker.expect("build -t apollo-generic-app-dependencies tmp") do
          files = Dir[File.join('tmp', '*')]
          expect(files).to include 'tmp/hook2.rb'
        end
        subject.run
      end

      it "builds a dependencies dockerfile that copies and runs he scripts" do
        docker.expect("build -t apollo-generic-app-dependencies tmp") do
          dockerfile = File.read(File.join('tmp', 'Dockerfile'))
          expected = File.read(File.join('spec', 'fixtures', 'app-dependencies-dockerfile-generic-with-library-install'))
          expect(dockerfile).to eq expected
        end
        subject.run
      end

      it "builds a normal before-startup" do
        docker.expect("build -t apollo-mars tmp") do
          startup = File.read(File.join('tmp', 'before_startup.bash'))
          expected = File.read(File.join('spec', 'fixtures', 'mars-before-startup'))
          expect(startup).to eq expected
        end
        subject.run
      end
    end

    context "with hooks for copying custom files" do
      let(:script_folder) { File.join(File.dirname(File.dirname(__FILE__)), 'fixtures', 'hooks') }
      before do
        script_folder = self.script_folder
        RubyYacht.configure do
          add_hooks script_folder: script_folder, container_type: :app do
            before :startup do
              copy_file "hook1.rb"
              command "cp /var/docker/hook1.rb /var/code/scripts/hook1.rb"
            end
          end
        end
      end

      it "copies the scripts to the tmp directory" do
        docker.expect("build -t apollo-mars tmp") do
          files = Dir[File.join('tmp', '*')]
          expect(files).to include 'tmp/hook1.rb'
        end
        subject.run
      end

      it "builds a before_startup script that copies the file" do
        docker.expect("build -t apollo-mars tmp") do
          dockerfile = File.read(File.join('tmp', 'before_startup.bash'))
          expected = File.read(File.join('spec', 'fixtures', 'mars-before-startup-with-custom-file-copy'))
          expect(dockerfile).to eq expected
        end
        subject.run
      end

      it "builds a dockerfile that copies the file to the /var/docker directory" do
        docker.expect("build -t apollo-mars tmp") do
          startup = File.read(File.join('tmp', 'Dockerfile'))
          expected = File.read(File.join('spec', 'fixtures', 'mars-dockerfile-with-custom-file-copy'))
          expect(startup).to eq expected
        end
        subject.run
      end
    end
    
    context "with an HTTPS repository" do
      before do
        RubyYacht.configuration.projects[0].repository_protocol = :https
      end
      
      it "builds a dependencies dockerfile that does checks the repository out over HTTPS" do
        docker.expect("build -t apollo-generic-app-dependencies tmp") do
          dockerfile = File.read(File.join('tmp', 'Dockerfile'))
          expected = File.read(File.join('spec', 'fixtures', 'app-dependencies-dockerfile-with-https-repository'))
          expect(dockerfile).to eq expected
        end
        subject.run
      end
    end
    
    context "with an app with no repository" do
      before do
        RubyYacht.configuration.projects[0].apps[0].repository_name = nil
        RubyYacht.configure do
          during :build_new_app do
            container_type :app
            command 'touch /var/code/app.bash'
          end
        end
      end
      
      it "builds a dependencies dockerfile that does not include that app" do
        docker.expect("build -t apollo-generic-app-dependencies tmp") do
          dockerfile = File.read(File.join('tmp', 'Dockerfile'))
          expected = File.read(File.join('spec', 'fixtures', 'app-dependencies-dockerfile-with-no-repository'))
          expect(dockerfile).to eq expected
        end
        subject.run
      end
      
      it "builds a dockerfile for that app that does not check it out" do
        docker.expect("build -t apollo-mars tmp") do
          dockerfile = File.read(File.join('tmp', 'Dockerfile'))
          expected = File.read(File.join('spec', 'fixtures', 'mars-dockerfile-with-no-repository'))
          expect(dockerfile).to eq expected
        end
        subject.run
      end
      
      it "builds a before-startup script for the app that does not check out the app" do
        docker.expect("build -t apollo-mars tmp") do
          file = File.read(File.join('tmp', 'before_startup.bash'))
          expected = File.read(File.join('spec', 'fixtures', 'mars-before-startup-with-no-repository'))
          expect(file).to eq expected
        end
        subject.run
      end
      
      it "builds a startup script for the app that creates the app before starting it" do
        docker.expect("build -t apollo-mars tmp") do
          file = File.read(File.join('tmp', 'startup.bash'))
          expected = File.read(File.join('spec', 'fixtures', 'mars-startup-with-no-repository'))
          expect(file).to eq expected
        end
        subject.run
      end
    end
  end

  describe 'helper methods' do
    describe 'set_image_settings' do
      context 'with an app folder' do
        it 'sets the templates, hook events, and hook servers' do
          app = RubyYacht::App.new
          subject.instance_eval { @app = app }
          subject.set_image_settings('app')
          expect(subject.instance_eval { @templates }).to eq ['Dockerfile', 'before_startup.bash', 'startup.bash']
          expect(subject.instance_eval { @hook_events }).to eq [:startup, :build_checkout, :build_new_app, :cleanup]
          expect(subject.instance_eval { @hook_servers }).to eq [app]
        end
      end

      context 'with an invalid folder' do
        it 'sets the templates, hook events, and hook servers' do
          subject.set_image_settings('invalid')
          expect(subject.instance_eval { @templates }).to eq ['Dockerfile']
          expect(subject.instance_eval { @hook_events }).to eq []
          expect(subject.instance_eval { @hook_servers }).to eq []
        end
      end
    end
  end
end