require 'spec_helper'

describe RubyYacht::Runner::Checkout do
  include DockerCommandSpec
  
  describe "command information" do
    specify { expect(described_class.command).to eq 'checkout' }
    specify { expect(described_class.description).to eq 'Check out a new branch for an app' }
    
    describe "command-line parsing" do
      let(:options) { subject.option_parser }
      it "gives a summary in the banner" do
        expect(options.banner).to eq "Usage: rspec checkout [options] [APP] [BRANCH]"
      end
      
      it "explains the options in the summary" do
        expect(options.summarize).to eq [
          "Options:\n",
          "    -p, --project PROJECT            The project with the app we are checking out the branch in. Default: apollo\n"
        ]
      end
      
      it "sets the project name from the command line" do
        RubyYacht.configure_second_test_project
        arguments = %w(-p jupiter venus issue1234)
        subject.option_parser.parse!(arguments)
        expect(subject.project_name).to eq :jupiter
        expect(arguments).to eq %w(venus issue1234)
      end
      
      it "takes the app and checkout from the command line" do
        subject.parse_positional_arguments(['venus', 'issue1234'])
        expect(subject.app_name).to eq 'venus'
        expect(subject.branch).to eq 'issue1234'
      end
    end
  end
  
  describe "run behavior" do
    before do
      subject.app_name = 'saturn'
      subject.branch = 'issue1234'
    end
    
    it "runs the commands to check out the branch and update the app" do
      docker.fail_unexpected_commands!
      docker.expect "exec apollo-saturn bash -c 'cd /var/code; git fetch; git checkout .; git checkout issue1234; git pull'"
      docker.expect "exec apollo-saturn /var/docker/before_startup.bash"
      docker.expect "restart apollo-saturn"
      subject.run
    end
    
    it "returns true" do
      expect(subject.run).to be_truthy
    end
    
    context "with an explicit project" do
      before do
        RubyYacht.configure_second_test_project
        subject.project_name = :jupiter
      end
      
      it "runs the command in that project" do
        subject.app_name = 'venus'
        docker.fail_unexpected_commands!
        docker.expect "exec jupiter-venus bash -c 'cd /var/code; git fetch; git checkout .; git checkout issue1234; git pull'"
        docker.expect "exec jupiter-venus /var/docker/before_startup.bash"
        docker.expect "restart jupiter-venus"
        subject.run
      end
    
      it "returns true" do
        expect(subject.run).to be_truthy
      end
    end
    
    context "with an invalid project" do
      before do
        subject.project_name = :foo
      end
      
      it "logs an error" do
        subject.run
        expect(@log_entries).to eq ['There is no project named foo']
      end
      
      it "returns false" do
        expect(subject.run).to be_falsey
      end
    end
    
    context "with no app" do
      before do
        subject.app_name = nil
      end
      
      it "logs an error" do
        subject.run
        expect(@log_entries).to eq [
          "You must provide an app name",
          "Run rspec help checkout for more information"
        ]
      end
      
      it "returns false" do
        expect(subject.run).to be_falsey
      end
    end
    
    context "with no branch" do
      before do
        subject.branch = nil
      end
      
      it "logs an error" do
        subject.run
        expect(@log_entries).to eq [
          "You must provide a branch",
          "Run rspec help checkout for more information"
        ]
      end
      
      it "returns false" do
        expect(subject.run).to be_falsey
      end
    end
  end
end
