require 'spec_helper'

describe RubyYacht::Runner::Command do
  class SampleCommand < RubyYacht::Runner::Command
    def self.command; 'sample_command'; end
    def self.description; 'Provides a sample command for testing'; end
  end
  
  describe "command information" do
    specify { expect { described_class.command }.to raise_error "Command not defined for RubyYacht::Runner::Command" }
    specify { expect { described_class.description }.to raise_error "Description not defined for RubyYacht::Runner::Command" }
    
    describe "option_parser" do
      it "takes the usage from the command information" do
        expect(SampleCommand.new.option_parser.banner).to eq "Usage: rspec sample_command\n\nProvides a sample command for testing"
      end
      
      it "does not have anything beyond the banner" do
        parser = SampleCommand.new.option_parser
        expect(parser.to_s).to eq parser.banner + "\n"
      end
    end
    
    describe "parse_positional_arguments" do
      it "does not modify the arguments" do
        arguments = %w(foo bar)
        subject.parse_positional_arguments(arguments)
        expect(arguments).to eq %w(foo bar)
      end
    end
  end
  
  describe "run behavior" do
    it "raises an error" do
      expect { subject.run }.to raise_error "Run method not defined for #{subject}"
    end
  end
  
  describe "helpers" do
    describe "short_script_name" do
      it "is the name of the invoked command" do
        expect(described_class.short_script_name).to eq "rspec"
      end
    end
    
    describe "projects" do
      it "provides the full list of projects" do
        RubyYacht.configure_test_project
        RubyYacht.configure_second_test_project
        expect(subject.projects.collect(&:name)).to eq [:apollo, :jupiter]
      end
    end
    
    describe "get_machine_info" do
      before do
        allow(subject).to receive(:backtick).with("/usr/local/bin/docker-machine inspect default -f {{System.Name}}").and_return("Jimmy\n")
      end

      context "with a docker machine" do
        it "gets the property from docker" do
          allow(subject).to receive(:docker_machine).and_return('/usr/local/bin/docker-machine')
          expect(subject.get_machine_info('System.Name')).to eq 'Jimmy'
        end
      end

      context "with no docker machine" do
        it "returns an empty string" do
          allow(subject).to receive(:docker_machine).and_return('')
          expect(subject.get_machine_info('System.Name')).to eq ''
        end
      end
    end

    describe 'docker_machine' do
      let(:docker_machine) { subject.docker_machine }

      before do
        RubyYacht.configuration.clear
      end

      context 'with docker-machine installed' do
        before do
          allow(subject).to receive(:backtick).with('which docker-machine').and_return("/usr/bin/docker-machine\n")
        end

        it 'is the path to docker-machine' do
          expect(docker_machine).to eq '/usr/bin/docker-machine'
        end
      end

      context 'with docker-machine not installed' do
        before do
          allow(subject).to receive(:backtick).with('which docker-machine').and_return('')
        end

        it 'is an empty string' do
          expect(docker_machine).to eq ''
        end
      end

      context 'with docker-machine disabled' do
        before do
          allow(subject).to receive(:backtick).with('which docker-machine').and_return('/usr/bin/docker-machine')
          RubyYacht.configuration.disable_docker_machine = true
        end

        it 'is an empty string' do
          expect(docker_machine).to eq ''
        end
      end
    end
    
    describe "logs" do
      it "logs the message to stdout" do
        allow($stdout).to receive(:puts)
        subject.log('Test Message')
        expect($stdout).to have_received(:puts).with('Test Message')
      end
    end
    
    describe "docker" do
      it "sends the command to docker" do
        allow(subject).to receive(:system)
        subject.docker "run hello-world"
        expect(subject).to have_received(:system).with('docker run hello-world')
      end
    end
    
    describe "system" do
      it "executes the command" do
        subject.system 'echo test > tmp/test'
        expect(File.read('tmp/test')).to eq "test\n"
        FileUtils.rm('tmp/test')
      end
    end
    
    describe "backtick" do
      it "returns the result of executing the command" do
        File.open('tmp/test', 'w') { |f| f.write('test contents') }
        value = subject.backtick('cat tmp/test')
        expect(value).to eq 'test contents'
        FileUtils.rm('tmp/test')
      end
    end
  end
end
