require 'spec_helper'

describe RubyYacht::Runner::RunContainers do
  include DockerCommandSpec
  
  describe "command information" do
    specify { expect(described_class.command).to eq 'run_containers' }
    specify { expect(described_class.description).to eq 'Runs containers for all the apps' }
  end
  
  describe "run behavior" do
    before do
      allow(subject).to receive(:remove_container)
    end
    
    it "runs the containers for all the apps" do
      docker.fail_unexpected_commands!
      docker.expect 'run -d --net=apollo --net-alias=apollo-mars --name=apollo-mars apollo-mars'
      docker.expect 'run -d --net=apollo --net-alias=apollo-saturn --name=apollo-saturn apollo-saturn'
      docker.expect 'run -d -p 80:80 --net=apollo --net-alias=apollo-nginx --name=apollo-nginx apollo-nginx'
      subject.run
    end
    
    it "tries to remove all the containers" do
      subject.run
      expect(subject).to have_received(:remove_container).with('apollo-mars')
      expect(subject).to have_received(:remove_container).with('apollo-saturn')
      expect(subject).to have_received(:remove_container).with('apollo-nginx')
    end
    
    it "returns true" do
      expect(subject.run).to be_truthy
    end
    
    context "with a local database" do
      before do
        database = RubyYacht::Database.new
        database.name = 'apollo'
        database.server_type = 'mysql'
        database.container_label = 'mysql'
        database.host = 'localhost'
        RubyYacht.configuration.projects.first.databases << database
      end
      
      it "runs the database container" do
        docker.expect 'run -d --net=apollo --net-alias=apollo-mysql --name=apollo-mysql apollo-mysql'
        subject.run
      end
    
      it "returns true" do
        expect(subject.run).to be_truthy
      end
    end
    
    context "with multiple projects" do
      before do
        RubyYacht.configure_second_test_project
      end
      
      it "runs the containers for both projects" do
        docker.fail_unexpected_commands!
        docker.expect 'run -d --net=apollo --net-alias=apollo-mars --name=apollo-mars apollo-mars'
        docker.expect 'run -d --net=apollo --net-alias=apollo-saturn --name=apollo-saturn apollo-saturn'
        docker.expect 'run -d -p 80:80 --net=apollo --net-alias=apollo-nginx --name=apollo-nginx apollo-nginx'
        docker.expect 'run -d --net=jupiter --net-alias=jupiter-venus --name=jupiter-venus jupiter-venus'
        docker.expect 'run -d --net=jupiter --net-alias=jupiter-saturn --name=jupiter-saturn jupiter-saturn'
        docker.expect 'run -d -p 80:80 --net=jupiter --net-alias=jupiter-web --name=jupiter-web jupiter-web'
        subject.run
      end
    
      it "returns true" do
        expect(subject.run).to be_truthy
      end
    end
    
    context "with DNS server config" do
      before do
        project = RubyYacht.configuration.projects.first
        project.dns_server = RubyYacht::DnsServer.new
        project.dns_server.servers = ['8.10.1.1', '8.10.1.2']
        project.dns_server.search_domains = ['apps.test.com', 'db.test.com']
      end
      
      it "gives the DNS information to the docker container" do
        docker.expect 'run -d --dns=8.10.1.1 --dns=8.10.1.2 --dns-search=apps.test.com --dns-search=db.test.com --net=apollo --net-alias=apollo-mars --name=apollo-mars apollo-mars'
        subject.run
      end
    
      it "returns true" do
        expect(subject.run).to be_truthy
      end
    end
    
    context "when checking out the code locally" do
      before do
        RubyYacht.configuration.projects.first.check_out_locally = true
      end
      
      it "maps the code volume to a local directory" do
        docker.expect 'run -d -v $PWD/../mars:/var/code --net=apollo --net-alias=apollo-mars --name=apollo-mars apollo-mars'
        docker.expect 'run -d -v $PWD/../saturn:/var/code --net=apollo --net-alias=apollo-saturn --name=apollo-saturn apollo-saturn'
        subject.run
      end
    
      it "returns true" do
        expect(subject.run).to be_truthy
      end
    end

    context "with an app with the same name as the project" do
      before do
        RubyYacht.configuration.projects.first.apps.first.name = :apollo
      end
    
      it "runs the containers for all the apps" do
        docker.fail_unexpected_commands!
        docker.expect 'run -d --net=apollo --net-alias=apollo --name=apollo apollo'
        docker.expect 'run -d --net=apollo --net-alias=apollo-saturn --name=apollo-saturn apollo-saturn'
        docker.expect 'run -d -p 80:80 --net=apollo --net-alias=apollo-nginx --name=apollo-nginx apollo-nginx'
        subject.run
      end
    end

    context 'with a special port for the web server' do
      before do
        RubyYacht.configuration.projects.first.web_servers.first.port = 3000
      end

      it 'maps the specified port on the host to port 80 in the container' do
        docker.expect 'run -d -p 3000:80 --net=apollo --net-alias=apollo-nginx --name=apollo-nginx apollo-nginx'
        subject.run
      end
    end
  end
  
  describe "remove_container" do
    it "sends the command to docker" do
      allow(IO).to receive(:popen)
      subject.remove_container('apollo-mars')
      expect(IO).to have_received(:popen).with('docker rm -f apollo-mars', err: :close)
    end
  end
end
