require 'spec_helper'

describe RubyYacht::Runner do
  before do
    allow(described_class).to receive(:puts)
    allow(described_class).to receive(:exit)
  end
  
  describe "load_default_commands" do
    before do
      RubyYacht::Runner.clear_commands
    end

    after do
      RubyYacht::Runner.clear_commands
    end

    context 'with the in_project flag set to true' do
      before do
        RubyYacht::Runner.load_default_commands(true)
      end

      it 'contains all the defined commands' do
        classes = %w(Help Build BuildImages RunContainers Services Checkout Shell UpdateHosts Implode)
        classes = classes.map { |klass| "RubyYacht::Runner::#{klass}" }
        expect(described_class.commands.map(&:name)).to eq classes
      end
    end

    context 'with the in_project flag set to false' do
      before do
        RubyYacht::Runner.load_default_commands(false)
      end

      it 'contains all the defined commands' do
        classes = %w(Help CreateNewProject)
        classes = classes.map { |klass| "RubyYacht::Runner::#{klass}" }
        expect(described_class.commands.map(&:name)).to eq classes
      end
    end
  end
  
  describe "arguments" do
    it "gets the command-line arguments" do
      expect(described_class.arguments).to eq ARGV
    end
  end
  
  describe "run" do
    let(:command) { RubyYacht::Runner::Checkout.new }
    before do
      allow(described_class).to receive(:arguments).and_return %w(checkout -p apollo mars issue1234)
      allow(command).to receive(:run)
      allow(RubyYacht::Runner::Checkout).to receive(:new).and_return(command)
      RubyYacht.configure_test_project
    end
    
    context "with a valid command" do
      context "with a successful command" do
        before do
          allow(command).to receive(:run).and_return(true)
          described_class.run
        end
        
        it "calls the command" do
          expect(command.project_name).to eq :apollo
          expect(command.app_name).to eq 'mars'
          expect(command.branch).to eq 'issue1234'
          expect(command).to have_received(:run)
        end
        
        it "does not exit" do
          expect(described_class).not_to have_received(:exit)
        end
      end
      
      context "with a failed command" do
        before do
          allow(command).to receive(:run).and_return(false)
          described_class.run
        end
        
        it "calls the command" do
          expect(command).to have_received(:run)
        end
        
        it "exits with a failure code" do
          expect(described_class).to have_received(:exit).with(1)
        end
      end
    end
    
    context "with a nil command" do
      before do
        allow(described_class).to receive(:arguments).and_return []
        described_class.run
      end
      
      it "logs a message" do
        invocation = $0
        expect(described_class).to have_received(:puts).with("You must provide a command to run")
        expect(described_class).to have_received(:puts).with("Run `#{invocation} help` for more information")
      end
      
      it "exits with a failure code" do
        expect(described_class).to have_received(:exit).with(1)
      end
    end
    
    context "with a blank command" do
      before do
        allow(described_class).to receive(:arguments).and_return ['']
        described_class.run
      end
      
      it "logs a message" do
        invocation = $0
        expect(described_class).to have_received(:puts).with("You must provide a command to run")
        expect(described_class).to have_received(:puts).with("Run `#{invocation} help` for more information")
      end
      
      it "exits with a failure code" do
        expect(described_class).to have_received(:exit).with(1)
      end
    end
    
    context "with an invalid command" do
      before do
        allow(described_class).to receive(:arguments).and_return %w(foo bar)
        described_class.run
      end
      
      it "logs a message" do
        invocation = $0
        expect(described_class).to have_received(:puts).with("Command not recognized: foo")
        expect(described_class).to have_received(:puts).with("Run `#{invocation} help` for more information")
      end
      
      it "exits with a failure code" do
        expect(described_class).to have_received(:exit).with(1)
      end
    end
  end
end