require 'spec_helper'

describe RubyYacht::Runner::Services do
  include DockerCommandSpec
  
  describe "command information" do
    specify { expect(described_class.command).to eq 'services' }
    specify { expect(described_class.description).to eq 'Start, stop, or restart containers' }
    
    it "has a banner with more information about the command" do
      usage = "Usage: rspec services [COMMAND]"
      usage += "\n\nStart, stop, or restart containers"
      usage += "\n\n[COMMAND] is required, and can be start, stop, or restart"
      expect(subject.option_parser.banner).to eq usage
    end
    
    it "gets the command from position arguments" do
      subject.parse_positional_arguments(["restart", "foo"])
      expect(subject.command).to eq 'restart'
    end
  end
  
  describe "run behavior" do
    before do
      subject.command = 'stop'
    end
    
    it "sends the command to each container" do
      docker.fail_unexpected_commands!
      docker.expect 'stop apollo-mars'
      docker.expect 'stop apollo-saturn'
      docker.expect 'stop apollo-nginx'
      subject.run
    end
    
    it "returns true" do
      expect(subject.run).to be_truthy
    end
    
    context "with a local database" do
      before do
        database = RubyYacht::Database.new
        database.name = 'apollo'
        database.server_type = 'mysql'
        database.container_label = 'mysql'
        database.host = 'localhost'
        RubyYacht.configuration.projects.first.databases << database
      end
      
      it "stops the database" do
        docker.expect 'stop apollo-mysql'
        subject.run
      end
    
      it "returns true" do
        expect(subject.run).to be_truthy
      end
    end
    
    context "with a docker machine" do
      before do
        allow(subject).to receive(:docker_machine).and_return '/usr/bin/docker-machine'
        allow(subject).to receive(:backtick).with('/usr/bin/docker-machine env default').and_return "# Test\nexport TEST_VAL=\"123\""
        ENV.delete('TEST_VAL')
      end
      
      after do
        ENV.delete('TEST_VAL')
      end
      
      context "with the start command" do
        before do
          subject.command = 'start'
        end
        
        it "starts the docker machine" do
          subject.run
          expect(@system_commands).to eq ['/usr/bin/docker-machine start default']
          expect(ENV['TEST_VAL']).to eq '123'
        end
    
        it "returns true" do
          expect(subject.run).to be_truthy
        end
      end
      
      context "with the stop command" do
        before do
          subject.command = 'stop'
        end
        
        it "does not send a message to docker machine" do
          subject.run
          expect(@system_commands).to eq []
          expect(ENV['TEST_VAL']).to be_nil
        end
    
        it "returns true" do
          expect(subject.run).to be_truthy
        end
      end
    end
    
    context "with no docker machine" do
      before do
        allow(subject).to receive(:docker_machine).and_return ''
        allow(subject).to receive(:backtick).with('docker-machine env default').and_return "# Test\nexport TEST_VAL=\"123\""
        ENV.delete('TEST_VAL')
      end
      
      after do
        ENV.delete('TEST_VAL')
      end
      
      context "with the start command" do
        before do
          subject.command = 'start'
        end
        
        it "does not start the docker machine" do
          subject.run
          expect(@system_commands).to eq []
          expect(ENV['TEST_VAL']).to be_nil
        end
    
        it "returns true" do
          expect(subject.run).to be_truthy
        end
      end
    end
    
    context "with no command" do
      before do
        subject.command = nil
      end
      
      it "logs error messages" do
        subject.run
        expect(@log_entries).to eq [
          "You must provide a command",
          "Run rspec help services for more information"
        ]
      end
      
      it "returns false" do
        expect(subject.run).to be_falsey
      end
    end
    
    context "with an invalid command" do
      before do
        subject.command = 'foo'
      end
      
      it "logs error messages" do
        subject.run
        expect(@log_entries).to eq [
          "foo is not a valid docker command",
          "Run rspec help services for more information"
        ]
      end
      
      it "returns false" do
        expect(subject.run).to be_falsey
      end
    end
  end
end
