require 'spec_helper'

describe RubyYacht::Runner::Shell do
  include DockerCommandSpec
  
  describe "command information" do
    specify { expect(described_class.command).to eq 'shell' }
    specify { expect(described_class.description).to eq 'Open a shell in a container' }
    
    describe "option parser" do
      it "has the usage banner" do
        expect(subject.option_parser.banner).to eq "Usage: rspec shell [options] [APP] [command]"
      end
      
      it "has the usage summary" do
        usage = ["\n"]
        usage << "The command is optional. If you omit it, this will open a bash shell\n"
        usage << "\n"
        usage << "Options:\n"
        usage << "    -p, --project PROJECT            The project with the app we are opening the sell in. Default: apollo\n"
        expect(subject.option_parser.summarize).to eq usage
      end
      
      it "takes the project from the command line" do
        arguments = ["-p", "jupiter", "venus"]
        subject.option_parser.parse!(arguments)
        expect(subject.project_name).to eq :jupiter
        expect(arguments).to eq ["venus"]
      end
    end
    
    describe "parse_positional_arguments" do
      it "takes the app from the command line" do
        subject.parse_positional_arguments(["venus", "rails", "c"])
        expect(subject.app_name).to eq "venus"
      end
      
      it "takes the command from the command line" do
        subject.parse_positional_arguments(["venus", "rails", "c"])
        expect(subject.command).to eq "rails c"
      end
      
      it "defaults the command to bash" do
        subject.parse_positional_arguments(["venus"])
        expect(subject.command).to eq "bash"
      end
    end
  end
  
  describe "run behavior" do
    before do
      subject.app_name = 'saturn'
      subject.command = 'rails c'
      RubyYacht.configure_second_test_project
      allow(subject).to receive :exec
    end
    
    describe "default_project" do
      it "is the first project defined" do
        expect(subject.default_project.name).to eq :apollo
      end
    end
    
    it "opens the shell in docker" do
      subject.run
      expect(subject).to have_received(:exec).with("docker exec -it apollo-saturn bash -c 'cd /var/code; TERM=xterm rails c'")
    end
    
    it "returns true" do
      expect(subject.run).to be_truthy
    end
    
    context "with no app" do
      before do
        subject.app_name = nil
      end
      
      it "logs an error" do
        subject.run
        expect(@log_entries).to eq [
          "You must provide an app name",
          "Run rspec help shell for more information"
        ]
      end
      
      it "returns false" do
        expect(subject.run).to be_falsey
      end
    end
    
    context "with a specific project" do
      before do
        subject.project_name = :jupiter
      end
      
      it "opens the shell in that project" do
        subject.run
        expect(subject).to have_received(:exec).with("docker exec -it jupiter-saturn bash -c 'cd /var/code; TERM=xterm rails c'")
      end
    
      it "returns true" do
        expect(subject.run).to be_truthy
      end
    end
    
    context "with an invalid project name" do
      before do
        subject.project_name = 'foo'
      end
      
      it "logs an error" do
        subject.run
        expect(@log_entries).to eq [
          "There is no project named foo"
        ]
      end
      
      it "returns false" do
        expect(subject.run).to be_falsey
      end
    end
  end
end
