require 'spec_helper'
require 'timecop'

describe RubyYacht::Runner::UpdateHosts do
  include DockerCommandSpec
  
  describe "command information" do
    specify { expect(described_class.command).to eq 'update_hosts' }
    specify { expect(described_class.description).to eq 'Add entries for your app domains to your hosts file' }
  end
  
  describe "current_host_contents" do
    it "is the contents of the hosts file" do
      expect(subject.current_host_contents).to eq File.read('/etc/hosts')
    end
  end
  
  describe "run behavior" do
    before do
      allow(subject).to receive("current_host_contents").and_return(<<-EOS.gsub(/^ +/, '')
      # Local information
      127.0.0.1 localhost
      
      # Database server
      
      40.1.30.10 db1.test.com
      EOS
      )
      
      allow(subject).to receive("get_machine_info").with('.Driver.IPAddress').and_return('192.168.1.27')
      Timecop.freeze
    end
    
    after do
      FileUtils.rm(File.join('tmp', 'hosts'))
      Timecop.return
    end
    
    it "adds the entries to the hosts file in the tmp folder" do
      subject.run
      expect(File.exist?(File.join('tmp', 'hosts'))).to be_truthy
      
      contents = File.read(File.join('tmp', 'hosts'))
      expect(contents).to eq <<-EOS.gsub(/^ +/, '').strip
      # Local information
      127.0.0.1 localhost
      
      # Database server
      
      40.1.30.10 db1.test.com
      
      # apollo docker containers
      192.168.1.27    apollo.test.com
      192.168.1.27    mars.apollo.test.com
      192.168.1.27    saturn.apollo.test.com
      EOS
      
    end
    
    it "copies the hosts file to a backup, and the tmp hosts file to the hosts file" do
      timestamp = Time.now.utc.strftime('%Y%m%d%H%M%S')
      allow(subject).to receive(:backtick).with('which sudo').and_return('/usr/bin/sudo')
      subject.run
      expect(@system_commands).to eq ["/usr/bin/sudo cp /etc/hosts /etc/hosts.#{timestamp}.backup", "/usr/bin/sudo cat tmp/hosts > /etc/hosts"]
    end
    
    it "returns true" do
      expect(subject.run).to be_truthy
    end
    
    context "with existing entries in the hosts file" do
      before do        
        allow(subject).to receive("current_host_contents").and_return(<<-EOS.gsub(/^ +/, '')
        # Local information
        127.0.0.1 localhost
        
        # Database server
        
        40.1.30.10 db1.test.com
        
        # apollo docker containers
        192.168.1.24 apollo.test.com
        192.168.1.24 mars.apollo.test.com
        192.168.1.24 venus.apollo.test.com
        EOS
        )
      end
      
      it "replaces the existing entries with the correct entries" do
        subject.run
        
        contents = File.read(File.join('tmp', 'hosts'))
        expect(contents).to eq <<-EOS.gsub(/^ +/, '').strip
        # Local information
        127.0.0.1 localhost
        
        # Database server
        
        40.1.30.10 db1.test.com
        
        # apollo docker containers
        192.168.1.27    apollo.test.com
        192.168.1.27    mars.apollo.test.com
        192.168.1.27    saturn.apollo.test.com
        EOS
      end
    end
    
    context "with no address from docker machine" do
      before do
        allow(subject).to receive("get_machine_info").with('.Driver.IPAddress').and_return('')
      end
      
      it "points the domains to localhost" do
        subject.run
        
        contents = File.read(File.join('tmp', 'hosts'))
        expect(contents).to eq <<-EOS.gsub(/^ +/, '').strip
        # Local information
        127.0.0.1 localhost
        
        # Database server
        
        40.1.30.10 db1.test.com
        
        # apollo docker containers
        127.0.0.1    apollo.test.com
        127.0.0.1    mars.apollo.test.com
        127.0.0.1    saturn.apollo.test.com
        EOS
      end
    end
    
    context "with multiple projects" do
      before do
        RubyYacht.configure_second_test_project
      end
      
      it "creates entries for both projects" do
        subject.run
        
        contents = File.read(File.join('tmp', 'hosts'))
        expect(contents).to eq <<-EOS.gsub(/^ +/, '').strip
        # Local information
        127.0.0.1 localhost
        
        # Database server
        
        40.1.30.10 db1.test.com
        
        # apollo docker containers
        192.168.1.27    apollo.test.com
        192.168.1.27    mars.apollo.test.com
        192.168.1.27    saturn.apollo.test.com
        
        # jupiter docker containers
        192.168.1.27    jupiter.test.com
        192.168.1.27    venus.jupiter.test.com
        192.168.1.27    saturn.jupiter.test.com
        EOS
      end
    end

    context "with an app with the same name as the project" do
      before do
        RubyYacht.configuration.projects.first.apps.first.name = :apollo
      end

      it "adds the entries to the hosts file in the tmp folder" do
        subject.run
        expect(File.exist?(File.join('tmp', 'hosts'))).to be_truthy
        
        contents = File.read(File.join('tmp', 'hosts'))
        expect(contents).to eq <<-EOS.gsub(/^ +/, '').strip
        # Local information
        127.0.0.1 localhost
        
        # Database server
        
        40.1.30.10 db1.test.com
        
        # apollo docker containers
        192.168.1.27    apollo.test.com
        192.168.1.27    saturn.apollo.test.com
        EOS
        
      end
    end
  end
end
