# This class stubs out commands that are sent to docker.
#
# This allows our tests to confirm that the correct commands
# are sent, as well as run blocks at the time commands are sent
# to check the context.
#
# To set an expectation on a command, call the `expect` method.
#
# By default, any commands that are not sent to `expect` are
# ignored. You can call `fail_unexpected_commands!` to raise an
# error instead.
class DockerStubber
  # This method creates a new docker stubber.
  def initialize()
    @blocks = []
    @fail_unexpected_commands = false
  end
  
  # The blocks to call when we receive commands.
  attr_accessor :blocks
  
  # This method tells the stubber to expect a command.
  #
  # === Parameters
  #
  # [command]  The command to expect
  # [block]    The block to call when the command is sent.
  def expect(command, &block)
    unless block_given?
      block = Proc.new {}
    end
    @blocks << [command, block]
  end
  
  # This method causes the stubber to raise an exception if it receives a
  # command it doesn't expect.
  def fail_unexpected_commands!
    @fail_unexpected_commands = true
  end
  
  # This method determines if we have received all the commands that it does not
  # expect.
  def has_received_all_commands?
    @blocks.none?
  end
  
  # This method is called when a command is sent to docker.
  #
  # === Parameters
  #
  # [command: String]    The command that was sent to docker.
  def receive(command)
    if @fail_unexpected_commands && command != (@blocks[0] || [])[0]
      message = "DockerStubber received unexpected command: #{command}"
      if @blocks[0]
        message += ". Expected #{@blocks[0][0]}"
      end
      raise message
    end
    
    entry = @blocks.find { |e| e[0] == command }
    if entry
      entry[1].call()
      @blocks.delete(entry)
    end
  end
end

# This module provides helpers for specs for our commands.
#
# This defines a `docker` method that gives a `DockerStubber`. It will also tell
# the `subject` command to stup to that docker instance, to stub all its logging,
# and will configure the test project.
module DockerCommandSpec
  def self.included(klass)    
    klass.let(:docker) { DockerStubber.new }
    klass.before do
      @log_entries = []
      @system_commands = []
      allow(subject).to receive(:docker) { |value| docker.receive(value) }
      allow(subject).to receive(:log) { |message| @log_entries << message }
      allow(subject).to receive(:system) { |command| @system_commands << command; 0 }
      allow(subject).to receive(:backtick).and_return('')
      
      FileUtils.rm Dir[File.join('tmp', '*')]
      RubyYacht.configure_test_project
    end
    
    klass.after do
      expect(docker).to have_received_all_commands
    end
  end
end