module IntegrationHelpers
  def containers
    `docker ps | grep "apollo\\|jupiter"`.split("\n").collect { |line| line.split(/\s+/).last }.sort
  end

  def images
    `docker images | grep "apollo\\|jupiter"`.split("\n").collect { |line| line.split(/\s+/).first }.sort
  end

  def run_command(command, environment: {}, capture_output: false)
    full_command = "ruby spec/integration/run.rb #{command}"
    environment.each do |key, value|
      full_command = "#{key}=#{value} #{full_command}"
    end
    if capture_output
      `#{full_command}`
    elsif ENV['VERBOSE_DOCKER'] == '1'
      system full_command
    else
      system "#{full_command} 1> log/integration_stdout.txt 2> log/integration_stderr.txt"
    end
  end
  
  def run_command_in_docker(image, command)
    `docker run --net=apollo --rm apollo-#{image} #{command}`
  end

  def wait_on_command_success(command)
    Timeout::timeout(60) do
      found = false
      while !found
        result = system "#{command} 1> /dev/null 2> /dev/null"
        found ||= !result.nil?
        break if found
        sleep 0.1
      end
    end
  end

  def wait_on_command_result(command, matchers)
    timeout = matchers.delete(:timeout) || 60
    Timeout::timeout(timeout) do
      found = false
      while !found
        result = `#{command} 2>&1`
        found ||= matchers[:contains] && result.include?(matchers[:contains])
        found ||= matchers[:does_not_contain] && !result.include?(matchers[:does_not_contain])
        break if found
        sleep 0.1
      end
    end
  end
  
  def start_database
    system "docker run -d --net=apollo --net-alias=apollo-database --name=apollo-database apollo-database > /dev/null"
    wait_on_command_result 'docker run --rm --net=apollo apollo-mars mysql -uapollo -ptestpass -hapollo-database', does_not_contain: 'ERROR'
  end

  def restore_hosts_file
    backup_path = Dir["/etc/hosts*.backup"].last
    if backup_path == nil
      raise "Missing backup file"
    end
    system "cat #{backup_path} > /etc/hosts"
    system "rm #{backup_path}"
  end
  
  def self.image_names
    images = %w(rails-app-dependencies saturn mars database web).map { |n| "apollo-#{n}"}
    images += %w(rails-app-dependencies saturn database web).map { |n| "jupiter-#{n}"}
    images += %w(apollo)
    images
  end

  def self.clear_docker_images
    image_names.each { |name| system "docker rmi #{name} 1> /dev/null 2> /dev/null" }
  end
  
  def self.clear_docker_containers
    image_names.each { |name| system "docker rm -f #{name} 1> /dev/null 2> /dev/null" }
  end
  
  def self.clear_checkout_folders
    %w(mars saturn).each do |app|
      if Dir.exist?("../#{app}")
        FileUtils.rm_rf("../#{app}")
      end
    end
  end
end